@@ -60,10 +60,10 @@ use tor_keymgr::{CTorClientKeystore, CTorServiceKeystore};
 
 use futures::StreamExt as _;
 use futures::lock::Mutex as AsyncMutex;
-use futures::task::SpawnExt;
 use std::net::IpAddr;
 use std::result::Result as StdResult;
 use std::sync::{Arc, Mutex};
+use tor_rtcompat::SpawnExt;
 
 use crate::err::ErrorDetail;
 use crate::{TorClientBuilder, status, util};
@@ -251,7 +251,9 @@ impl InertTorClient {
 
                 let native_store =
                     ArtiNativeKeystore::from_path_and_mistrust(&key_store_dir, permissions)?;
-                info!("Using keystore from {key_store_dir:?}");
+                // Should only log fs paths at debug level or lower,
+                // unless they're part of a diagnostic message.
+                debug!("Using keystore from {key_store_dir:?}");
 
                 Box::new(native_store)
             }
@@ -351,14 +353,6 @@ impl InertTorClient {
         feature = "experimental-api",
         feature = "keymgr"
     ))]
-    #[cfg_attr(
-        docsrs,
-        doc(cfg(all(
-            feature = "onion-service-client",
-            feature = "experimental-api",
-            feature = "keymgr"
-        )))
-    )]
     pub fn rotate_service_discovery_key(
         &self,
         selector: KeystoreSelector,
@@ -483,6 +477,34 @@ impl InertTorClient {
             action: "get key manager handle",
         })?)
     }
+
+    /// Create (but do not launch) a new
+    /// [`OnionService`](tor_hsservice::OnionService)
+    /// using the given configuration.
+    ///
+    /// See [`TorClient::create_onion_service`].
+    #[cfg(feature = "onion-service-service")]
+    #[instrument(skip_all, level = "trace")]
+    pub fn create_onion_service(
+        &self,
+        config: &TorClientConfig,
+        svc_config: tor_hsservice::OnionServiceConfig,
+    ) -> crate::Result<tor_hsservice::OnionService> {
+        let keymgr = self.keymgr.as_ref().ok_or(ErrorDetail::KeystoreRequired {
+            action: "create onion service",
+        })?;
+
+        let (state_dir, mistrust) = config.state_dir()?;
+        let state_dir =
+            self::StateDirectory::new(state_dir, mistrust).map_err(ErrorDetail::StateAccess)?;
+
+        Ok(tor_hsservice::OnionService::builder()
+            .config(svc_config)
+            .keymgr(keymgr.clone())
+            .state_dir(state_dir)
+            .build()
+            .map_err(ErrorDetail::OnionServiceSetup)?)
+    }
 }
 
 /// Preferences for whether a [`TorClient`] should bootstrap on its own or not.
@@ -615,7 +637,6 @@ impl StreamPrefs {
     /// When this option is set, we will only pick exit relays that
     /// have an IP address that matches the country in our GeoIP database.
     #[cfg(feature = "geoip")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "geoip")))]
     pub fn exit_country(&mut self, country_code: CountryCode) -> &mut Self {
         self.country_code = Some(country_code);
         self
@@ -841,6 +862,7 @@ impl<R: Runtime> TorClient<R> {
 
     /// Implementation of `create_unbootstrapped`, split out in order to avoid manually specifying
     /// double error conversions.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn create_inner(
         runtime: R,
         config: &TorClientConfig,
@@ -1241,6 +1263,7 @@ impl<R: Runtime> TorClient<R> {
     /// This is split out from `reconfigure` so we can do the all-or-nothing
     /// check without recursion. the caller to this method must hold the
     /// `reconfigure_lock`.
+    #[instrument(level = "trace", skip_all)]
     fn reconfigure_inner(
         &self,
         new_config: &TorClientConfig,
@@ -1581,6 +1604,7 @@ impl<R: Runtime> TorClient<R> {
     /// Perform a remote DNS reverse lookup with the provided IP address.
     ///
     /// On success, return a list of hostnames.
+    #[instrument(level = "trace", skip_all)]
     pub async fn resolve_ptr_with_prefs(
         &self,
         addr: IpAddr,
@@ -1730,6 +1754,8 @@ impl<R: Runtime> TorClient<R> {
 
     /// Try to launch an onion service with a given configuration.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// This onion service will not actually handle any requests on its own: you
     /// will need to
     /// pull [`RendRequest`](tor_hsservice::RendRequest) objects from the returned stream,
@@ -1746,10 +1772,22 @@ impl<R: Runtime> TorClient<R> {
     pub fn launch_onion_service(
         &self,
         config: tor_hsservice::OnionServiceConfig,
-    ) -> crate::Result<(
-        Arc<tor_hsservice::RunningOnionService>,
-        impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
-    )> {
+    ) -> crate::Result<
+        Option<(
+            Arc<tor_hsservice::RunningOnionService>,
+            impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
+        )>,
+    > {
+        let nickname = config.nickname();
+
+        if !config.enabled() {
+            info!(
+                nickname=%nickname,
+                "Skipping onion service because it was disabled in the config"
+            );
+            return Ok(None);
+        }
+
         let keymgr = self
             .inert_client
             .keymgr
@@ -1767,16 +1805,14 @@ impl<R: Runtime> TorClient<R> {
             .state_dir(state_dir)
             .build()
             .map_err(ErrorDetail::LaunchOnionService)?;
-        let (service, stream) = service
+        Ok(service
             .launch(
                 self.runtime.clone(),
                 self.dirmgr.clone().upcast_arc(),
                 self.hs_circ_pool.clone(),
                 Arc::clone(&self.path_resolver),
             )
-            .map_err(ErrorDetail::LaunchOnionService)?;
-
-        Ok((service, stream))
+            .map_err(ErrorDetail::LaunchOnionService)?)
     }
 
     /// Try to launch an onion service with a given configuration and provided
@@ -1784,6 +1820,8 @@ impl<R: Runtime> TorClient<R> {
     /// associated `HsIdKeypair`  in this `TorClient`'s `KeyMgr`, then this operation
     /// fails rather than overwriting the existing key.
     ///
+    /// Returns `Ok(None)` if the service specified is disabled in the config.
+    ///
     /// The specified `HsIdKeypair` will be inserted in the primary keystore.
     ///
     /// **Important**: depending on the configuration of your
@@ -1809,10 +1847,12 @@ impl<R: Runtime> TorClient<R> {
         &self,
         config: tor_hsservice::OnionServiceConfig,
         id_keypair: HsIdKeypair,
-    ) -> crate::Result<(
-        Arc<tor_hsservice::RunningOnionService>,
-        impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
-    )> {
+    ) -> crate::Result<
+        Option<(
+            Arc<tor_hsservice::RunningOnionService>,
+            impl futures::Stream<Item = tor_hsservice::RendRequest> + use<R>,
+        )>,
+    > {
         let nickname = config.nickname();
         let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
         let selector = KeystoreSelector::Primary;
@@ -1857,14 +1897,6 @@ impl<R: Runtime> TorClient<R> {
         feature = "experimental-api",
         feature = "keymgr"
     ))]
-    #[cfg_attr(
-        docsrs,
-        doc(cfg(all(
-            feature = "onion-service-client",
-            feature = "experimental-api",
-            feature = "keymgr"
-        )))
-    )]
     pub fn generate_service_discovery_key(
         &self,
         selector: KeystoreSelector,
@@ -2029,6 +2061,7 @@ impl<R: Runtime> TorClient<R> {
     /// The behaviour is not specified if you create the `OnionService` with
     /// `create_onion_service()` using one [`TorClientConfig`],
     /// but launch it using a `TorClient` generated from a different `TorClientConfig`.
+    // TODO #2249: Look into this behaviour more, and possibly error if there is a different config.
     #[cfg(feature = "onion-service-service")]
     #[instrument(skip_all, level = "trace")]
     pub fn create_onion_service(
@@ -2036,20 +2069,7 @@ impl<R: Runtime> TorClient<R> {
         svc_config: tor_hsservice::OnionServiceConfig,
     ) -> crate::Result<tor_hsservice::OnionService> {
         let inert_client = InertTorClient::new(config)?;
-        let keymgr = inert_client.keymgr.ok_or(ErrorDetail::KeystoreRequired {
-            action: "create onion service",
-        })?;
-
-        let (state_dir, mistrust) = config.state_dir()?;
-        let state_dir =
-            self::StateDirectory::new(state_dir, mistrust).map_err(ErrorDetail::StateAccess)?;
-
-        Ok(tor_hsservice::OnionService::builder()
-            .config(svc_config)
-            .keymgr(keymgr)
-            .state_dir(state_dir)
-            .build()
-            .map_err(ErrorDetail::OnionServiceSetup)?)
+        inert_client.create_onion_service(config, svc_config)
     }
 
     /// Return a current [`status::BootstrapStatus`] describing how close this client
@@ -2098,6 +2118,12 @@ impl<R: Runtime> TorClient<R> {
         // dropped—which is what we want.
         self.statemgr.wait_for_unlock()
     }
+
+    /// Getter for keymgr.
+    #[cfg(feature = "onion-service-cli-extra")]
+    pub fn keymgr(&self) -> crate::Result<&KeyMgr> {
+        self.inert_client.keymgr()
+    }
 }
 
 /// Monitor `dormant_mode` and enable/disable periodic tasks as applicable
@@ -2158,7 +2184,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
