@@ -3,27 +3,27 @@
 use std::time::Duration;
 
 use super::AbstractChannelFactory;
-use super::{select, AbstractChannel, Pending, Sending};
+use super::{AbstractChannel, Pending, Sending, select};
 use crate::{ChannelConfig, Dormancy, Error, Result};
 
 use futures::FutureExt;
 use std::result::Result as StdResult;
-use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
+use std::sync::atomic::{AtomicU64, Ordering};
 use tor_async_utils::oneshot;
 use tor_basic_utils::RngExt as _;
 use tor_cell::chancell::msg::PaddingNegotiate;
 use tor_config::PaddingLevel;
 use tor_error::{error_report, internal, into_internal};
 use tor_linkspec::{HasRelayIds, ListByRelayIds, RelayIds};
-use tor_netdir::{params::NetParameters, params::CHANNEL_PADDING_TIMEOUT_UPPER_BOUND};
+use tor_netdir::{params::CHANNEL_PADDING_TIMEOUT_UPPER_BOUND, params::NetParameters};
+use tor_proto::ChannelPaddingInstructions;
+use tor_proto::channel::ChannelPaddingInstructionsUpdates;
 use tor_proto::channel::kist::{KistMode, KistParams};
 use tor_proto::channel::padding::Parameters as PaddingParameters;
 use tor_proto::channel::padding::ParametersBuilder as PaddingParametersBuilder;
-use tor_proto::channel::ChannelPaddingInstructionsUpdates;
-use tor_proto::ChannelPaddingInstructions;
 use tor_units::{BoundedInt32, IntegerMilliseconds};
-use tracing::info;
+use tracing::{info, instrument};
 use void::{ResultVoidExt as _, Void};
 
 #[cfg(test)]
@@ -496,6 +496,7 @@ impl<C: AbstractChannelFactory> MgrState<C> {
 
     /// Upgrade the pending channel identified by its `handle` by replacing it with a new open
     /// `channel`.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) fn upgrade_pending_channel_to_open(
         &self,
         handle: PendingChannelHandle,
@@ -862,7 +863,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -870,6 +871,8 @@ mod test {
     use super::*;
     use crate::factory::BootstrapReporter;
     use async_trait::async_trait;
+    #[cfg(feature = "relay")]
+    use safelog::Sensitive;
     use std::sync::{Arc, Mutex};
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
@@ -908,7 +911,7 @@ mod test {
         #[cfg(feature = "relay")]
         async fn build_channel_using_incoming(
             &self,
-            _peer: std::net::SocketAddr,
+            _peer: Sensitive<std::net::SocketAddr>,
             _stream: Self::Stream,
             _memquota: ChannelAccount,
         ) -> Result<Arc<Self::Channel>> {
