@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -30,7 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
-#![deny(clippy::unchecked_duration_subtraction)]
+#![deny(clippy::unchecked_time_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -47,25 +47,27 @@
 // TODO #1645 (either remove this, or decide to have it everywhere)
 #![cfg_attr(not(all(feature = "full", feature = "experimental")), allow(unused))]
 
-use build::CircuitBuilder;
-use mgr::{AbstractCirc, AbstractCircBuilder};
+use build::TunnelBuilder;
+use mgr::{AbstractTunnel, AbstractTunnelBuilder};
 use tor_basic_utils::retry::RetryDelay;
 use tor_chanmgr::ChanMgr;
+use tor_dircommon::fallback::FallbackList;
 use tor_error::{error_report, warn_report};
 use tor_guardmgr::RetireCircuits;
 use tor_linkspec::ChanTarget;
 use tor_netdir::{DirEvent, NetDir, NetDirProvider, Timeliness};
-use tor_proto::circuit::{CircParameters, ClientCirc, UniqId};
+use tor_proto::circuit::UniqId;
+use tor_proto::client::circuit::CircParameters;
 use tor_rtcompat::Runtime;
 
 #[cfg(any(feature = "specific-relay", feature = "hs-common"))]
 use tor_linkspec::IntoOwnedChanTarget;
 
-use futures::task::SpawnExt;
 use futures::StreamExt;
 use std::sync::{Arc, Mutex, Weak};
 use std::time::{Duration, Instant};
-use tracing::{debug, info, trace, warn};
+use tor_rtcompat::SpawnExt;
+use tracing::{debug, info, instrument, trace, warn};
 
 #[cfg(feature = "testing")]
 pub use config::test_config::TestConfig;
@@ -82,6 +84,7 @@ mod mgr;
 mod mocks;
 mod preemptive;
 pub mod timeouts;
+mod tunnel;
 mod usage;
 
 // Can't apply `visibility` to modules.
@@ -95,8 +98,17 @@ cfg_if::cfg_if! {
 
 pub use err::Error;
 pub use isolation::IsolationToken;
-use tor_guardmgr::fallback::FallbackList;
 pub use tor_guardmgr::{ClockSkewEvents, GuardMgrConfig, SkewEstimate};
+pub use tunnel::{
+    ClientDataTunnel, ClientDirTunnel, ClientOnionServiceDataTunnel, ClientOnionServiceDirTunnel,
+    ClientOnionServiceIntroTunnel, ServiceOnionServiceDataTunnel, ServiceOnionServiceDirTunnel,
+    ServiceOnionServiceIntroTunnel,
+};
+#[cfg(feature = "conflux")]
+pub use tunnel::{
+    ClientMultiPathDataTunnel, ClientMultiPathOnionServiceDataTunnel,
+    ServiceMultiPathOnionServiceDataTunnel,
+};
 pub use usage::{TargetPort, TargetPorts};
 
 pub use config::{
@@ -105,9 +117,9 @@ pub use config::{
 };
 
 use crate::isolation::StreamIsolation;
-use crate::mgr::CircProvenance;
+use crate::mgr::TunnelProvenance;
 use crate::preemptive::PreemptiveCircuitPredictor;
-use usage::TargetCircUsage;
+use usage::TargetTunnelUsage;
 
 use safelog::sensitive as sv;
 #[cfg(feature = "geoip")]
@@ -161,7 +173,7 @@ impl<'a> From<&'a NetDir> for DirInfo<'a> {
 }
 impl<'a> DirInfo<'a> {
     /// Return a set of circuit parameters for this DirInfo.
-    fn circ_params(&self, usage: &TargetCircUsage) -> Result<CircParameters> {
+    fn circ_params(&self, usage: &TargetTunnelUsage) -> Result<CircParameters> {
         use tor_netdir::params::NetParameters;
         // We use a common function for both cases here to be sure that
         // we look at the defaults from NetParameters code.
@@ -172,7 +184,7 @@ impl<'a> DirInfo<'a> {
         };
         match usage {
             #[cfg(feature = "hs-common")]
-            TargetCircUsage::HsCircBase { .. } => {
+            TargetTunnelUsage::HsCircBase { .. } => {
                 build::onion_circparams_from_netparams(net_params)
             }
             _ => build::exit_circparams_from_netparams(net_params),
@@ -189,7 +201,7 @@ impl<'a> DirInfo<'a> {
 /// a set of ports; directory circuits were made to talk to directory caches.
 ///
 /// This is a "handle"; clones of it share state.
-pub struct CircMgr<R: Runtime>(Arc<CircMgrInner<build::CircuitBuilder<R>, R>>);
+pub struct CircMgr<R: Runtime>(Arc<CircMgrInner<build::TunnelBuilder<R>, R>>);
 
 impl<R: Runtime> CircMgr<R> {
     /// Construct a new circuit manager.
@@ -214,8 +226,10 @@ impl<R: Runtime> CircMgr<R> {
 
     /// Return a circuit suitable for sending one-hop BEGINDIR streams,
     /// launching it if necessary.
-    pub async fn get_or_launch_dir(&self, netdir: DirInfo<'_>) -> Result<Arc<ClientCirc>> {
-        self.0.get_or_launch_dir(netdir).await
+    #[instrument(level = "trace", skip_all)]
+    pub async fn get_or_launch_dir(&self, netdir: DirInfo<'_>) -> Result<ClientDirTunnel> {
+        let tunnel = self.0.get_or_launch_dir(netdir).await?;
+        Ok(tunnel.into())
     }
 
     /// Return a circuit suitable for exiting to all of the provided
@@ -223,6 +237,7 @@ impl<R: Runtime> CircMgr<R> {
     ///
     /// If the list of ports is empty, then the chosen circuit will
     /// still end at _some_ exit.
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch_exit(
         &self,
         netdir: DirInfo<'_>, // TODO: This has to be a NetDir.
@@ -231,8 +246,9 @@ impl<R: Runtime> CircMgr<R> {
         // TODO GEOIP: this cannot be stabilised like this, since Cargo features need to be
         //             additive. The function should be refactored to be builder-like.
         #[cfg(feature = "geoip")] country_code: Option<CountryCode>,
-    ) -> Result<Arc<ClientCirc>> {
-        self.0
+    ) -> Result<ClientDataTunnel> {
+        let tunnel = self
+            .0
             .get_or_launch_exit(
                 netdir,
                 ports,
@@ -240,20 +256,22 @@ impl<R: Runtime> CircMgr<R> {
                 #[cfg(feature = "geoip")]
                 country_code,
             )
-            .await
+            .await?;
+        Ok(tunnel.into())
     }
 
     /// Return a circuit to a specific relay, suitable for using for direct
     /// (one-hop) directory downloads.
     ///
     /// This could be used, for example, to download a descriptor for a bridge.
-    #[cfg_attr(docsrs, doc(cfg(feature = "specific-relay")))]
     #[cfg(feature = "specific-relay")]
+    #[instrument(level = "trace", skip_all)]
     pub async fn get_or_launch_dir_specific<T: IntoOwnedChanTarget>(
         &self,
         target: T,
-    ) -> Result<Arc<ClientCirc>> {
-        self.0.get_or_launch_dir_specific(target).await
+    ) -> Result<ClientDirTunnel> {
+        let tunnel = self.0.get_or_launch_dir_specific(target).await?;
+        Ok(tunnel.into())
     }
 
     /// Launch the periodic daemon tasks required by the manager to function properly.
@@ -261,6 +279,7 @@ impl<R: Runtime> CircMgr<R> {
     /// Returns a set of [`TaskHandle`]s that can be used to manage the daemon tasks.
     //
     // NOTE(eta): The ?Sized on D is so we can pass a trait object in.
+    #[instrument(level = "trace", skip_all)]
     pub fn launch_background_tasks<D, S>(
         self: &Arc<Self>,
         runtime: &R,
@@ -326,6 +345,7 @@ impl<R: Runtime> CircMgr<R> {
     /// The actual behavior here will depend on the value of `how`.
     ///
     /// Returns whether any of the circuit pools should be cleared.
+    #[instrument(level = "trace", skip_all)]
     pub fn reconfigure<CFG: CircMgrConfig>(
         &self,
         new_config: &CFG,
@@ -366,21 +386,21 @@ impl<R: Runtime> CircMgr<R> {
     /// Return a reference to the associated CircuitBuilder that this CircMgr
     /// will use to create its circuits.
     #[cfg(feature = "experimental-api")]
-    pub fn builder(&self) -> &CircuitBuilder<R> {
+    pub fn builder(&self) -> &TunnelBuilder<R> {
         CircMgrInner::builder(&self.0)
     }
 }
 
 /// Internal object used to implement CircMgr, which allows for mocking.
 #[derive(Clone)]
-pub(crate) struct CircMgrInner<B: AbstractCircBuilder<R> + 'static, R: Runtime> {
+pub(crate) struct CircMgrInner<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> {
     /// The underlying circuit manager object that implements our behavior.
-    mgr: Arc<mgr::AbstractCircMgr<B, R>>,
+    mgr: Arc<mgr::AbstractTunnelMgr<B, R>>,
     /// A preemptive circuit predictor, for, uh, building circuits preemptively.
     predictor: Arc<Mutex<PreemptiveCircuitPredictor>>,
 }
 
-impl<R: Runtime> CircMgrInner<CircuitBuilder<R>, R> {
+impl<R: Runtime> CircMgrInner<TunnelBuilder<R>, R> {
     /// Construct a new circuit manager.
     ///
     /// # Usage note
@@ -414,7 +434,7 @@ impl<R: Runtime> CircMgrInner<CircuitBuilder<R>, R> {
 
         let storage_handle = storage.create_handle(PARETO_TIMEOUT_DATA_KEY);
 
-        let builder = build::CircuitBuilder::new(
+        let builder = build::TunnelBuilder::new(
             runtime.clone(),
             chanmgr,
             config.path_rules().clone(),
@@ -428,7 +448,7 @@ impl<R: Runtime> CircMgrInner<CircuitBuilder<R>, R> {
     }
 }
 
-impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
+impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// Generic implementation for [`CircMgrInner::new`]
     pub(crate) fn new_generic<CFG: CircMgrConfig>(
         config: &CFG,
@@ -443,7 +463,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         guardmgr.set_filter(config.path_rules().build_guard_filter());
 
         let mgr =
-            mgr::AbstractCircMgr::new(builder, runtime.clone(), config.circuit_timing().clone());
+            mgr::AbstractTunnelMgr::new(builder, runtime.clone(), config.circuit_timing().clone());
 
         CircMgrInner {
             mgr: Arc::new(mgr),
@@ -456,6 +476,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// Returns a set of [`TaskHandle`]s that can be used to manage the daemon tasks.
     //
     // NOTE(eta): The ?Sized on D is so we can pass a trait object in.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) fn launch_background_tasks<D, S>(
         self: &Arc<Self>,
         runtime: &R,
@@ -528,9 +549,10 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
 
     /// Return a circuit suitable for sending one-hop BEGINDIR streams,
     /// launching it if necessary.
-    pub(crate) async fn get_or_launch_dir(&self, netdir: DirInfo<'_>) -> Result<Arc<B::Circ>> {
-        self.expire_circuits();
-        let usage = TargetCircUsage::Dir;
+    #[instrument(level = "trace", skip_all)]
+    pub(crate) async fn get_or_launch_dir(&self, netdir: DirInfo<'_>) -> Result<Arc<B::Tunnel>> {
+        self.expire_circuits().await;
+        let usage = TargetTunnelUsage::Dir;
         self.mgr.get_or_launch(&usage, netdir).await.map(|(c, _)| c)
     }
 
@@ -539,6 +561,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// If the list of ports is empty, then the chosen circuit will
     /// still end at _some_ exit.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn get_or_launch_exit(
         &self,
         netdir: DirInfo<'_>, // TODO: This has to be a NetDir.
@@ -547,8 +570,8 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         // TODO GEOIP: this cannot be stabilised like this, since Cargo features need to be
         //             additive. The function should be refactored to be builder-like.
         #[cfg(feature = "geoip")] country_code: Option<CountryCode>,
-    ) -> Result<Arc<B::Circ>> {
-        self.expire_circuits();
+    ) -> Result<Arc<B::Tunnel>> {
+        self.expire_circuits().await;
         let time = Instant::now();
         {
             let mut predictive = self.predictor.lock().expect("preemptive lock poisoned");
@@ -570,7 +593,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         let ports = ports.iter().map(Clone::clone).collect();
         #[cfg(not(feature = "geoip"))]
         let country_code = None;
-        let usage = TargetCircUsage::Exit {
+        let usage = TargetTunnelUsage::Exit {
             ports,
             isolation,
             country_code,
@@ -583,14 +606,14 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// (one-hop) directory downloads.
     ///
     /// This could be used, for example, to download a descriptor for a bridge.
-    #[cfg_attr(docsrs, doc(cfg(feature = "specific-relay")))]
     #[cfg(feature = "specific-relay")]
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn get_or_launch_dir_specific<T: IntoOwnedChanTarget>(
         &self,
         target: T,
-    ) -> Result<Arc<B::Circ>> {
-        self.expire_circuits();
-        let usage = TargetCircUsage::DirSpecificTarget(target.to_owned());
+    ) -> Result<Arc<B::Tunnel>> {
+        self.expire_circuits().await;
+        let usage = TargetTunnelUsage::DirSpecificTarget(target.to_owned());
         self.mgr
             .get_or_launch(&usage, DirInfo::Nothing)
             .await
@@ -602,6 +625,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// The actual behavior here will depend on the value of `how`.
     ///
     /// Returns whether any of the circuit pools should be cleared.
+    #[instrument(level = "trace", skip_all)]
     pub(crate) fn reconfigure<CFG: CircMgrConfig>(
         &self,
         new_config: &CFG,
@@ -671,6 +695,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// dangling.
     ///
     /// This is a daemon task: it runs indefinitely in the background.
+    #[instrument(level = "trace", skip_all)]
     async fn keep_circmgr_params_updated<D>(
         mut events: impl futures::Stream<Item = DirEvent> + Unpin,
         circmgr: Weak<Self>,
@@ -706,6 +731,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// Exit when we notice that `circmgr` or `dirmgr` has been dropped.
     ///
     /// This is a daemon task: it runs indefinitely in the background.
+    #[instrument(level = "trace", skip_all)]
     async fn continually_launch_timeout_testing_circuits<D>(
         mut sched: TaskSchedule<R>,
         circmgr: Weak<Self>,
@@ -716,7 +742,10 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         while sched.next().await.is_some() {
             if let (Some(cm), Some(dm)) = (Weak::upgrade(&circmgr), Weak::upgrade(&dirmgr)) {
                 if let Ok(netdir) = dm.netdir(Timeliness::Unchecked) {
-                    if let Err(e) = cm.launch_timeout_testing_circuit_if_appropriate(&netdir) {
+                    if let Err(e) = cm
+                        .launch_timeout_testing_circuit_if_appropriate(&netdir)
+                        .await
+                    {
                         warn_report!(e, "Problem launching a timeout testing circuit");
                     }
                     let delay = netdir
@@ -746,21 +775,21 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// This function is invoked periodically from
     /// `continually_launch_timeout_testing_circuits`.
-    fn launch_timeout_testing_circuit_if_appropriate(&self, netdir: &NetDir) -> Result<()> {
+    async fn launch_timeout_testing_circuit_if_appropriate(&self, netdir: &NetDir) -> Result<()> {
         if !self.mgr.peek_builder().learning_timeouts() {
             return Ok(());
         }
         // We expire any too-old circuits here, so they don't get
         // counted towards max_circs.
-        self.expire_circuits();
+        self.expire_circuits().await;
         let max_circs: u64 = netdir
             .params()
             .cbt_max_open_circuits_for_testing
             .try_into()
             .expect("Out-of-bounds result from BoundedInt32");
-        if (self.mgr.n_circs() as u64) < max_circs {
+        if (self.mgr.n_tunnels() as u64) < max_circs {
             // Actually launch the circuit!
-            let usage = TargetCircUsage::TimeoutTesting;
+            let usage = TargetTunnelUsage::TimeoutTesting;
             let dirinfo = netdir.into();
             let mgr = Arc::clone(&self.mgr);
             debug!("Launching a circuit to test build times.");
@@ -780,6 +809,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// This is a daemon task: it runs indefinitely in the background.
     #[allow(clippy::cognitive_complexity)] // because of tracing
+    #[instrument(level = "trace", skip_all)]
     async fn update_persistent_state<S>(
         mut sched: TaskSchedule<R>,
         circmgr: Weak<Self>,
@@ -860,6 +890,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// This would be better handled entirely within `tor-circmgr`, like
     /// other daemon tasks.
+    #[instrument(level = "trace", skip_all)]
     async fn continually_preemptively_build_circuits<D>(
         mut sched: TaskSchedule<R>,
         circmgr: Weak<Self>,
@@ -906,6 +937,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// This function is invoked periodically from
     /// `continually_preemptively_build_circuits()`.
     #[allow(clippy::cognitive_complexity)]
+    #[instrument(level = "trace", skip_all)]
     async fn launch_circuits_preemptively(
         &self,
         netdir: DirInfo<'_>,
@@ -918,7 +950,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
             (preemptive.predict(&path_config), threshold)
         };
 
-        if self.mgr.n_circs() >= threshold {
+        if self.mgr.n_tunnels() >= threshold {
             return Ok(());
         }
         let mut n_created = 0_usize;
@@ -930,11 +962,11 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
         let results = futures::future::join_all(futures).await;
         for (i, result) in results.into_iter().enumerate() {
             match result {
-                Ok((_, CircProvenance::NewlyCreated)) => {
+                Ok((_, TunnelProvenance::NewlyCreated)) => {
                     debug!("Preeemptive circuit was created for {:?}", circs[i]);
                     n_created += 1;
                 }
-                Ok((_, CircProvenance::Preexisting)) => {
+                Ok((_, TunnelProvenance::Preexisting)) => {
                     trace!("Circuit already existed created for {:?}", circs[i]);
                 }
                 Err(e) => {
@@ -973,17 +1005,18 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// Used to implement onion service clients and services.
     #[cfg(feature = "hs-common")]
+    #[instrument(level = "trace", skip_all)]
     pub(crate) async fn launch_hs_unmanaged<T>(
         &self,
         planned_target: Option<T>,
         dir: &NetDir,
         stem_kind: HsCircStemKind,
         circ_kind: Option<HsCircKind>,
-    ) -> Result<Arc<B::Circ>>
+    ) -> Result<B::Tunnel>
     where
         T: IntoOwnedChanTarget,
     {
-        let usage = TargetCircUsage::HsCircBase {
+        let usage = TargetTunnelUsage::HsCircBase {
             compatible_with_target: planned_target.map(IntoOwnedChanTarget::to_owned),
             stem_kind,
             circ_kind,
@@ -1030,14 +1063,16 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     ///
     /// Expired circuits are not closed while they still have users,
     /// but they are no longer given out for new requests.
-    fn expire_circuits(&self) {
+    async fn expire_circuits(&self) {
         // TODO: I would prefer not to call this at every request, but
         // it should be fine for now.  (At some point we may no longer
         // need this, or might not need to call it so often, now that
         // our circuit expiration runs on scheduled timers via
         // spawn_expiration_task.)
         let now = self.mgr.peek_runtime().now();
-        self.mgr.expire_circs(now);
+
+        // TODO: Use return value here to implement the above TODO.
+        let _next_expiration = self.mgr.expire_tunnels(now).await;
     }
 
     /// Mark every circuit that we have launched so far as unsuitable for
@@ -1048,13 +1083,13 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     /// TODO: we may want to expose this eventually.  If we do, we should
     /// be very clear that you don't want to use it haphazardly.
     pub(crate) fn retire_all_circuits(&self) {
-        self.mgr.retire_all_circuits();
+        self.mgr.retire_all_tunnels();
     }
 
     /// If `circ_id` is the unique identifier for a circuit that we're
     /// keeping track of, don't give it out for any future requests.
-    pub(crate) fn retire_circ(&self, circ_id: &<B::Circ as AbstractCirc>::Id) {
-        let _ = self.mgr.take_circ(circ_id);
+    pub(crate) fn retire_circ(&self, circ_id: &<B::Tunnel as AbstractTunnel>::Id) {
+        let _ = self.mgr.take_tunnel(circ_id);
     }
 
     /// Return a stream of events about our estimated clock skew; these events
@@ -1096,7 +1131,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> CircMgrInner<B, R> {
     }
 }
 
-impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> Drop for CircMgrInner<B, R> {
+impl<B: AbstractTunnelBuilder<R> + 'static, R: Runtime> Drop for CircMgrInner<B, R> {
     fn drop(&mut self) {
         match self.store_persistent_state() {
             Ok(true) => info!("Flushed persistent state at exit."),
@@ -1117,7 +1152,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -1137,7 +1172,7 @@ mod test {
         let fb = FallbackList::from([]);
         let di: DirInfo<'_> = (&fb).into();
 
-        let p1 = di.circ_params(&TargetCircUsage::Dir).unwrap();
+        let p1 = di.circ_params(&TargetTunnelUsage::Dir).unwrap();
         assert!(!p1.extend_by_ed25519_id);
 
         // Now try with a directory and configured parameters.
@@ -1151,7 +1186,7 @@ mod test {
         }
         let netdir = dir.unwrap_if_sufficient().unwrap();
         let di: DirInfo<'_> = (&netdir).into();
-        let p2 = di.circ_params(&TargetCircUsage::Dir).unwrap();
+        let p2 = di.circ_params(&TargetTunnelUsage::Dir).unwrap();
         assert!(p2.extend_by_ed25519_id);
 
         // Now try with a bogus circwindow value.
@@ -1165,7 +1200,7 @@ mod test {
         }
         let netdir = dir.unwrap_if_sufficient().unwrap();
         let di: DirInfo<'_> = (&netdir).into();
-        let p2 = di.circ_params(&TargetCircUsage::Dir).unwrap();
+        let p2 = di.circ_params(&TargetTunnelUsage::Dir).unwrap();
         assert!(p2.extend_by_ed25519_id);
     }
 
