@@ -9,8 +9,6 @@
 //! For TP-based keys, that involves deriving [`HsTimePeriodKeySpecifier`]
 //! and adding a call to `remove_if_expired!` in [`expire_publisher_keys`].
 
-use tor_keymgr::{CTorPath, CTorServicePath};
-
 use crate::{internal_prelude::*, list_expired_keys_for_service};
 
 /// Keys that are used by publisher, which relate to our HS and a TP
@@ -46,27 +44,19 @@ define_derive_deftly! {
 #[deftly(role = "KP_hs_id")]
 #[deftly(summary = "Public part of the identity key")]
 #[deftly(keypair_specifier = "HsIdKeypairSpecifier")]
-#[deftly(ctor_path = "hsid_public_key_specifier_ctor_path")]
+#[deftly(ctor_path = "HsIdPublicKey")]
 /// The public part of the identity key of the service.
 pub struct HsIdPublicKeySpecifier {
     /// The nickname of the  hidden service.
     nickname: HsNickname,
 }
 
-/// The `CTorPath` of HsIdPublicKeySpecifier
-fn hsid_public_key_specifier_ctor_path(spec: &HsIdPublicKeySpecifier) -> CTorPath {
-    CTorPath::Service {
-        nickname: spec.nickname.clone(),
-        path: CTorServicePath::PublicKey,
-    }
-}
-
 #[derive(Deftly, PartialEq, Debug, Constructor)]
 #[derive_deftly(KeySpecifier)]
 #[deftly(prefix = "hss")]
 #[deftly(role = "KS_hs_id")]
 #[deftly(summary = "Long-term identity keypair")]
-#[deftly(ctor_path = "hsid_keypair_key_specifier_ctor_path")]
+#[deftly(ctor_path = "HsIdKeypair")]
 /// The long-term identity keypair of the service.
 pub struct HsIdKeypairSpecifier {
     /// The nickname of the  hidden service.
@@ -79,14 +69,6 @@ impl From<&HsIdPublicKeySpecifier> for HsIdKeypairSpecifier {
     }
 }
 
-/// The `CTorPath` of HsIdKeypairKeySpecifier
-fn hsid_keypair_key_specifier_ctor_path(spec: &HsIdKeypairSpecifier) -> CTorPath {
-    CTorPath::Service {
-        nickname: spec.nickname.clone(),
-        path: CTorServicePath::PrivateKey,
-    }
-}
-
 #[derive(Deftly, PartialEq, Debug, Constructor)]
 #[derive_deftly(KeySpecifier, HsTimePeriodKeySpecifier)]
 #[deftly(prefix = "hss")]
