@@ -54,21 +54,33 @@ pub mod vote;
 #[cfg(feature = "build_docs")]
 mod build;
 
+#[cfg(feature = "parse2")]
+use {
+    crate::parse2::{self, ArgumentStream}, //
+};
+
+#[cfg(feature = "parse2")]
+pub use {
+    parse2_impls::ProtoStatusesNetdocParseAccumulator, //
+};
+
 use crate::doc::authcert::{AuthCert, AuthCertKeyIds};
 use crate::parse::keyword::Keyword;
 use crate::parse::parser::{Section, SectionRules, SectionRulesBuilder};
 use crate::parse::tokenize::{Item, ItemResult, NetDocReader};
 use crate::types::misc::*;
 use crate::util::PeekableIterator;
-use crate::{Error, NetdocErrorKind as EK, Pos, Result};
-use std::collections::{HashMap, HashSet};
+use crate::{Error, KeywordEncodable, NetdocErrorKind as EK, NormalItemArgument, Pos, Result};
+use std::collections::{BTreeSet, HashMap, HashSet};
+use std::fmt::{self, Display};
 use std::result::Result as StdResult;
+use std::str::FromStr;
 use std::sync::Arc;
 use std::{net, result, time};
 use tor_error::{HasKind, internal};
 use tor_protover::Protocols;
 
-use bitflags::bitflags;
+use derive_deftly::{Deftly, define_derive_deftly};
 use digest::Digest;
 use std::sync::LazyLock;
 use tor_checkable::{ExternallySigned, timed::TimerangeBound};
@@ -87,11 +99,9 @@ ns_export_each_flavor! {
 }
 
 ns_export_each_variety! {
-    ty: RouterStatus, Header;
+    ty: RouterStatus, Preamble;
 }
 
-use void::ResultVoidExt as _;
-
 #[deprecated]
 #[cfg(feature = "ns_consensus")]
 pub use PlainConsensus as NsConsensus;
@@ -105,79 +115,161 @@ pub use UncheckedPlainConsensus as UncheckedNsConsensus;
 #[cfg(feature = "ns_consensus")]
 pub use UnvalidatedPlainConsensus as UnvalidatedNsConsensus;
 
+#[cfg(feature = "ns-vote")]
+pub use rs::RouterStatusMdDigestsVote;
+
+/// `publiscation` field in routerstatus entry intro item other than in votes
+///
+/// Two arguments which are both ignored.
+/// This used to be an ISO8601 timestamp in anomalous two-argument format.
+///
+/// Nowadays, according to the spec, it can be a dummy value.
+/// So it can be a unit type.
+///
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:r>,
+/// except in votes which use [`Iso8601TimeSp`] instead.
+///
+/// **Not the same as** the `published` item:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd, Default)]
+#[allow(clippy::exhaustive_structs)]
+pub struct IgnoredPublicationTimeSp;
+
 /// The lifetime of a networkstatus document.
 ///
 /// In a consensus, this type describes when the consensus may safely
 /// be used.  In a vote, this type describes the proposed lifetime for a
 /// consensus.
-#[derive(Clone, Debug)]
+///
+/// Aggregate of three netdoc preamble fields.
+#[derive(Clone, Debug, Deftly)]
+#[derive_deftly(Lifetime)]
+#[cfg_attr(feature = "parse2", derive_deftly(NetdocParseableFields))]
 pub struct Lifetime {
-    /// Time at which the document becomes valid
-    valid_after: time::SystemTime,
-    /// Time after which there is expected to be a better version
-    /// of this consensus
-    fresh_until: time::SystemTime,
-    /// Time after which this consensus is expired.
+    /// `valid-after` --- Time at which the document becomes valid
     ///
-    /// (In practice, Tor clients will keep using documents for a while
-    /// after this expiration time, if no better one can be found.)
-    valid_until: time::SystemTime,
-}
-
-impl Lifetime {
-    /// Construct a new Lifetime.
-    pub fn new(
-        valid_after: time::SystemTime,
-        fresh_until: time::SystemTime,
-        valid_until: time::SystemTime,
-    ) -> Result<Self> {
-        if valid_after < fresh_until && fresh_until < valid_until {
-            Ok(Lifetime {
-                valid_after,
-                fresh_until,
-                valid_until,
-            })
-        } else {
-            Err(EK::InvalidLifetime.err())
-        }
-    }
-    /// Return time when this consensus first becomes valid.
+    /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
     ///
     /// (You might see a consensus a little while before this time,
     /// since voting tries to finish up before the.)
-    pub fn valid_after(&self) -> time::SystemTime {
-        self.valid_after
-    }
-    /// Return time when this consensus is no longer fresh.
+    #[cfg_attr(feature = "parse2", deftly(netdoc(single_arg)))]
+    valid_after: Iso8601TimeSp,
+    /// `fresh-until` --- Time after which there is expected to be a better version
+    /// of this consensus
+    ///
+    /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
     ///
     /// You can use the consensus after this time, but there is (or is
     /// supposed to be) a better one by this point.
-    pub fn fresh_until(&self) -> time::SystemTime {
-        self.fresh_until
-    }
-    /// Return the time when this consensus is no longer valid.
+    #[cfg_attr(feature = "parse2", deftly(netdoc(single_arg)))]
+    fresh_until: Iso8601TimeSp,
+    /// `valid-until` --- Time after which this consensus is expired.
+    ///
+    /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
     ///
     /// You should try to get a better consensus after this time,
     /// though it's okay to keep using this one if no more recent one
     /// can be found.
-    pub fn valid_until(&self) -> time::SystemTime {
-        self.valid_until
-    }
-    /// Return true if this consensus is officially valid at the provided time.
-    pub fn valid_at(&self, when: time::SystemTime) -> bool {
-        self.valid_after <= when && when <= self.valid_until
-    }
+    #[cfg_attr(feature = "parse2", deftly(netdoc(single_arg)))]
+    valid_until: Iso8601TimeSp,
+}
 
-    /// Return the voting period implied by this lifetime.
-    ///
-    /// (The "voting period" is the amount of time in between when a consensus first
-    /// becomes valid, and when the next consensus is expected to become valid)
-    pub fn voting_period(&self) -> time::Duration {
-        let valid_after = self.valid_after();
-        let fresh_until = self.fresh_until();
-        fresh_until
-            .duration_since(valid_after)
-            .expect("Mis-formed lifetime")
+define_derive_deftly! {
+    /// Bespoke derive for `Lifetime`, for `new` and accessors
+    Lifetime:
+
+    impl Lifetime {
+        /// Construct a new Lifetime.
+        pub fn new(
+            $( $fname: time::SystemTime, )
+        ) -> Result<Self> {
+            // Make this now because otherwise literal `valid_after` here in the body
+            // has the wrong span - the compiler refuses to look at the argument.
+            // But we can refer to the field names.
+            let self_ = Lifetime {
+                $( $fname: $fname.into(), )
+            };
+            if self_.valid_after < self_.fresh_until && self_.fresh_until < self_.valid_until {
+                Ok(self_)
+            } else {
+                Err(EK::InvalidLifetime.err())
+            }
+        }
+      $(
+        ${fattrs doc}
+        pub fn $fname(&self) -> time::SystemTime {
+            *self.$fname
+        }
+      )
+        /// Return true if this consensus is officially valid at the provided time.
+        pub fn valid_at(&self, when: time::SystemTime) -> bool {
+            *self.valid_after <= when && when <= *self.valid_until
+        }
+
+        /// Return the voting period implied by this lifetime.
+        ///
+        /// (The "voting period" is the amount of time in between when a consensus first
+        /// becomes valid, and when the next consensus is expected to become valid)
+        pub fn voting_period(&self) -> time::Duration {
+            let valid_after = self.valid_after();
+            let fresh_until = self.fresh_until();
+            fresh_until
+                .duration_since(valid_after)
+                .expect("Mis-formed lifetime")
+        }
+    }
+}
+use derive_deftly_template_Lifetime;
+
+/// A single consensus method
+///
+/// These are integers, but we don't do arithmetic on them.
+///
+/// As defined here:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:consensus-methods>
+/// <https://spec.torproject.org/dir-spec/computing-consensus.html#flavor:microdesc>
+///
+/// As used in a `consensus-method` item:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:consensus-method>
+#[derive(Debug, Clone, Default, Eq, PartialEq, Ord, PartialOrd, Hash, Copy)] //
+#[derive(derive_more::From, derive_more::Into, derive_more::Display, derive_more::FromStr)]
+pub struct ConsensusMethod(u32);
+impl NormalItemArgument for ConsensusMethod {}
+
+/// A set of consensus methods
+///
+/// Implements `ItemValueParseable` as required for `consensus-methods`,
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:consensus-methods>
+///
+/// There is also [`consensus_methods_comma_separated`] for `m` lines in votes.
+#[derive(Debug, Clone, Default, Eq, PartialEq)]
+#[cfg_attr(feature = "parse2", derive(Deftly), derive_deftly(ItemValueParseable))]
+#[non_exhaustive]
+pub struct ConsensusMethods {
+    /// Consensus methods.
+    pub methods: BTreeSet<ConsensusMethod>,
+}
+
+/// Module for use with parse2's `with`, to parse one argument of comma-separated consensus methods
+///
+/// As found in an `m` item in a vote:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:m>
+#[cfg(feature = "parse2")]
+pub mod consensus_methods_comma_separated {
+    use super::*;
+    use parse2::ArgumentError as AE;
+    use std::result::Result;
+
+    /// Parse
+    pub fn from_args<'s>(args: &mut ArgumentStream<'s>) -> Result<ConsensusMethods, AE> {
+        let mut methods = BTreeSet::new();
+        for ent in args.next().ok_or(AE::Missing)?.split(',') {
+            let ent = ent.parse().map_err(|_| AE::Invalid)?;
+            if !methods.insert(ent) {
+                return Err(AE::Invalid);
+            }
+        }
+        Ok(ConsensusMethods { methods })
     }
 }
 
@@ -189,6 +281,13 @@ impl Lifetime {
 ///
 /// A `NetParams<i32>` is part of the validated directory manager configuration,
 /// where it is built (in the builder-pattern sense) from a transparent HashMap.
+///
+/// As found in `params` in a network status:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:params>
+///
+/// The same syntax is also used, and this type used for parsing, in various other places,
+/// for example routerstatus entry `w` items (bandwith weights):
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:w>
 #[derive(Debug, Clone, Default, Eq, PartialEq)]
 pub struct NetParams<T> {
     /// Map from keys to values.
@@ -246,14 +345,23 @@ where
 
 /// A list of subprotocol versions that implementors should/must provide.
 ///
+/// This struct represents a pair of (optional) items:
+/// `recommended-FOO-protocols` and `required-FOO-protocols`.
+///
 /// Each consensus has two of these: one for relays, and one for clients.
+///
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:required-relay-protocols>
 #[derive(Debug, Clone, Default, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct ProtoStatus {
     /// Set of protocols that are recommended; if we're missing a protocol
     /// in this list we should warn the user.
+    ///
+    /// `recommended-client-protocols` or `recommended-relay-protocols`
     recommended: Protocols,
     /// Set of protocols that are required; if we're missing a protocol
     /// in this list we should refuse to start.
+    ///
+    /// `required-client-protocols` or `required-relay-protocols`
     required: Protocols,
 }
 
@@ -318,7 +426,11 @@ impl HasKind for ProtocolSupportError {
 
 /// A set of recommended and required protocols when running
 /// in various scenarios.
-#[derive(Clone, Debug, PartialEq, serde::Serialize, serde::Deserialize)]
+///
+/// Represents the collection of four items: `{recommended,required}-{client,relay}-protocols`.
+///
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:required-relay-protocols>
+#[derive(Clone, Debug, Default, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct ProtoStatuses {
     /// Lists of recommended and required subprotocol versions for clients
     client: ProtoStatus,
@@ -339,6 +451,8 @@ impl ProtoStatuses {
 }
 
 /// A recognized 'flavor' of consensus document.
+///
+/// <https://spec.torproject.org/dir-spec/computing-consensus.html#flavors>
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, Ord, PartialOrd)]
 #[non_exhaustive]
 pub enum ConsensusFlavor {
@@ -412,8 +526,10 @@ pub struct SharedRandVal([u8; 32]);
 
 /// A shared-random value produced by the directory authorities,
 /// along with meta-information about that value.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Deftly)]
 #[non_exhaustive]
+#[cfg_attr(feature = "parse2", derive_deftly(ItemValueParseable))]
+#[cfg_attr(feature = "encode", derive_deftly(ItemValueEncodable))]
 pub struct SharedRandStatus {
     /// How many authorities revealed shares that contributed to this value.
     pub n_reveals: u8,
@@ -428,7 +544,7 @@ pub struct SharedRandStatus {
     /// The time when this SharedRandVal becomes (or became) the latest.
     ///
     /// (This is added per proposal 342, assuming that gets accepted.)
-    pub timestamp: Option<time::SystemTime>,
+    pub timestamp: Option<Iso8601TimeNoSp>,
 }
 
 /// Description of an authority's identity and address.
@@ -453,69 +569,6 @@ pub struct DirSource {
     pub or_port: u16,
 }
 
-bitflags! {
-    /// A set of recognized directory flags on a single relay.
-    ///
-    /// These flags come from a consensus directory document, and are
-    /// used to describe what the authorities believe about the relay.
-    /// If the document contained any flags that we _didn't_ recognize,
-    /// they are not listed in this type.
-    ///
-    /// The bit values used to represent the flags have no meaning;
-    /// they may change between releases of this crate.  Relying on their
-    /// values may void your semver guarantees.
-    #[derive(Clone, Copy, Debug)]
-    pub struct RelayFlags: u16 {
-        /// Is this a directory authority?
-        const AUTHORITY = (1<<0);
-        /// Is this relay marked as a bad exit?
-        ///
-        /// Bad exits can be used as intermediate relays, but not to
-        /// deliver traffic.
-        const BAD_EXIT = (1<<1);
-        /// Is this relay marked as an exit for weighting purposes?
-        const EXIT = (1<<2);
-        /// Is this relay considered "fast" above a certain threshold?
-        const FAST = (1<<3);
-        /// Is this relay suitable for use as a guard relay?
-        ///
-        /// Clients choose their their initial relays from among the set
-        /// of Guard relays.
-        const GUARD = (1<<4);
-        /// Does this relay participate on the onion service directory
-        /// ring?
-        const HSDIR = (1<<5);
-        /// Set if this relay is considered "middle only", not suitable to run
-        /// as an exit or guard relay.
-        ///
-        /// Note that this flag is only used by authorities as part of
-        /// the voting process; clients do not and should not act
-        /// based on whether it is set.
-        const MIDDLE_ONLY = (1<<6);
-        /// If set, there is no consensus for the ed25519 key for this relay.
-        const NO_ED_CONSENSUS = (1<<7);
-        /// Is this relay considered "stable" enough for long-lived circuits?
-        const STABLE = (1<<8);
-        /// Set if the authorities are requesting a fresh descriptor for
-        /// this relay.
-        const STALE_DESC = (1<<9);
-        /// Set if this relay is currently running.
-        ///
-        /// This flag can appear in votes, but in consensuses, every relay
-        /// is assumed to be running.
-        const RUNNING = (1<<10);
-        /// Set if this relay is considered "valid" -- allowed to be on
-        /// the network.
-        ///
-        /// This flag can appear in votes, but in consensuses, every relay
-        /// is assumed to be valid.
-        const VALID = (1<<11);
-        /// Set if this relay supports a currently recognized version of the
-        /// directory protocol.
-        const V2DIR = (1<<12);
-    }
-}
-
 /// Recognized weight fields on a single relay in a consensus
 #[non_exhaustive]
 #[derive(Debug, Clone, Copy)]
@@ -860,6 +913,21 @@ where
     }
 }
 
+impl FromStr for SharedRandVal {
+    type Err = Error;
+    fn from_str(s: &str) -> Result<Self> {
+        let val: B64 = s.parse()?;
+        let val = SharedRandVal(val.into_array()?);
+        Ok(val)
+    }
+}
+impl Display for SharedRandVal {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        Display::fmt(&B64::from(Vec::from(self.0)), f)
+    }
+}
+impl NormalItemArgument for SharedRandVal {}
+
 impl SharedRandStatus {
     /// Parse a current or previous shared rand value from a given
     /// SharedRandPreviousValue or SharedRandCurrentValue.
@@ -875,12 +943,9 @@ impl SharedRandStatus {
             }
         }
         let n_reveals: u8 = item.parse_arg(0)?;
-        let val: B64 = item.parse_arg(1)?;
-        let value = SharedRandVal(val.into_array()?);
+        let value: SharedRandVal = item.parse_arg(1)?;
         // Added in proposal 342
-        let timestamp = item
-            .parse_optional_arg::<Iso8601TimeNoSp>(2)?
-            .map(Into::into);
+        let timestamp = item.parse_optional_arg::<Iso8601TimeNoSp>(2)?;
         Ok(SharedRandStatus {
             n_reveals,
             value,
@@ -895,7 +960,7 @@ impl SharedRandStatus {
 
     /// Return the timestamp (if any) associated with this `SharedRandValue`.
     pub fn timestamp(&self) -> Option<std::time::SystemTime> {
-        self.timestamp
+        self.timestamp.map(|t| t.0)
     }
 }
 
@@ -953,59 +1018,6 @@ impl ConsensusVoterInfo {
     }
 }
 
-impl std::str::FromStr for RelayFlags {
-    type Err = void::Void;
-    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        Ok(match s {
-            "Authority" => RelayFlags::AUTHORITY,
-            "BadExit" => RelayFlags::BAD_EXIT,
-            "Exit" => RelayFlags::EXIT,
-            "Fast" => RelayFlags::FAST,
-            "Guard" => RelayFlags::GUARD,
-            "HSDir" => RelayFlags::HSDIR,
-            "MiddleOnly" => RelayFlags::MIDDLE_ONLY,
-            "NoEdConsensus" => RelayFlags::NO_ED_CONSENSUS,
-            "Stable" => RelayFlags::STABLE,
-            "StaleDesc" => RelayFlags::STALE_DESC,
-            "Running" => RelayFlags::RUNNING,
-            "Valid" => RelayFlags::VALID,
-            "V2Dir" => RelayFlags::V2DIR,
-            _ => RelayFlags::empty(),
-        })
-    }
-}
-
-impl RelayFlags {
-    /// Parse a relay-flags entry from an "s" line.
-    fn from_item(item: &Item<'_, NetstatusKwd>) -> Result<RelayFlags> {
-        if item.kwd() != NetstatusKwd::RS_S {
-            return Err(
-                Error::from(internal!("Wrong keyword {:?} for S line", item.kwd()))
-                    .at_pos(item.pos()),
-            );
-        }
-        // These flags are implicit.
-        let mut flags: RelayFlags = RelayFlags::RUNNING | RelayFlags::VALID;
-
-        let mut prev: Option<&str> = None;
-        for s in item.args() {
-            if let Some(p) = prev {
-                if p >= s {
-                    // Arguments out of order.
-                    return Err(EK::BadArgument
-                        .at_pos(item.pos())
-                        .with_msg("Flags out of order"));
-                }
-            }
-            let fl = s.parse().void_unwrap();
-            flags |= fl;
-            prev = Some(s);
-        }
-
-        Ok(flags)
-    }
-}
-
 impl Default for RelayWeight {
     fn default() -> RelayWeight {
         RelayWeight::Unmeasured(0)
@@ -1022,8 +1034,15 @@ impl RelayWeight {
             );
         }
 
-        let params: NetParams<u32> = item.args_as_str().parse()?;
+        let params = item.args_as_str().parse()?;
 
+        Self::from_net_params(&params).map_err(|e| e.at_pos(item.pos()))
+    }
+
+    /// Parse a routerweight from partially-parsed `w` line in the form of a `NetParams`
+    ///
+    /// This function is the common part shared between `parse2` and `parse`.
+    fn from_net_params(params: &NetParams<u32>) -> Result<RelayWeight> {
         let bw = params.params.get("Bandwidth");
         let unmeas = params.params.get("Unmeasured");
 
@@ -1035,9 +1054,119 @@ impl RelayWeight {
         match unmeas {
             None | Some(0) => Ok(RelayWeight::Measured(bw)),
             Some(1) => Ok(RelayWeight::Unmeasured(bw)),
-            _ => Err(EK::BadArgument
-                .at_pos(item.pos())
-                .with_msg("unmeasured value")),
+            _ => Err(EK::BadArgument.with_msg("unmeasured value")),
+        }
+    }
+}
+
+/// `parse2` impls for types in this module
+///
+/// Separate module to save on repeated `cfg` and for a separate namespace.
+#[cfg(feature = "parse2")]
+mod parse2_impls {
+    use super::*;
+    use parse2::ArgumentError as AE;
+    use parse2::ErrorProblem as EP;
+    use parse2::{ArgumentStream, ItemArgumentParseable, ItemValueParseable};
+    use parse2::{KeywordRef, NetdocParseableFields, UnparsedItem};
+    use paste::paste;
+    use std::result::Result;
+
+    /// Implements `NetdocParseableFields` for `ProtoStatuses`
+    ///
+    /// We have this macro so that it's impossible to write things like
+    /// ```text
+    ///      ProtoStatuses {
+    ///          client: ProtoStatus {
+    ///              recommended: something something recommended_relay_versions something,
+    /// ```
+    ///
+    /// (The structure of `ProtoStatuses` means the normal parse2 derive won't work for it.
+    /// Note the bug above: the recommended *relay* version info is put in the *client* field.
+    /// Preventing this bug must involve: avoiding writing twice the field name elements,
+    /// such as `relay` and `client`, during this kind of construction/conversion.)
+    macro_rules! impl_proto_statuses { { $( $rr:ident $cr:ident; )* } => { paste! {
+        #[derive(Deftly)]
+        #[derive_deftly(NetdocParseableFields)]
+        // Only ProtoStatusesParseNetdocParseAccumulator is exposed.
+        #[allow(unreachable_pub)]
+        pub struct ProtoStatusesParseHelper {
+            $(
+                #[deftly(netdoc(default))]
+                [<$rr _ $cr _protocols>]: Protocols,
+            )*
+        }
+
+        /// Partially parsed `ProtoStatuses`
+        pub use ProtoStatusesParseHelperNetdocParseAccumulator
+            as ProtoStatusesNetdocParseAccumulator;
+
+        impl NetdocParseableFields for ProtoStatuses {
+            type Accumulator = ProtoStatusesNetdocParseAccumulator;
+            fn is_item_keyword(kw: KeywordRef<'_>) -> bool {
+                ProtoStatusesParseHelper::is_item_keyword(kw)
+            }
+            fn accumulate_item(
+                acc: &mut Self::Accumulator,
+                item: UnparsedItem<'_>,
+            ) -> Result<(), EP> {
+                ProtoStatusesParseHelper::accumulate_item(acc, item)
+            }
+            fn finish(acc: Self::Accumulator) -> Result<Self, EP> {
+                let parse = ProtoStatusesParseHelper::finish(acc)?;
+                let mut out = ProtoStatuses::default();
+                $(
+                    out.$cr.$rr = parse.[< $rr _ $cr _protocols >];
+                )*
+                Ok(out)
+            }
+        }
+    } } }
+
+    impl_proto_statuses! {
+        required client;
+        required relay;
+        recommended client;
+        recommended relay;
+    }
+
+    impl ItemValueParseable for NetParams<i32> {
+        fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            item.args_copy()
+                .into_remaining()
+                .parse()
+                .map_err(item.invalid_argument_handler("parameters"))
+        }
+    }
+
+    impl ItemValueParseable for RelayWeight {
+        fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            (|| {
+                let params = item.args_copy().into_remaining().parse()?;
+                Self::from_net_params(&params)
+            })()
+            .map_err(item.invalid_argument_handler("weights"))
+        }
+    }
+
+    impl ItemValueParseable for rs::Version {
+        fn from_unparsed(mut item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            item.args_mut()
+                .into_remaining()
+                .parse()
+                .map_err(item.invalid_argument_handler("version"))
+        }
+    }
+
+    impl ItemArgumentParseable for IgnoredPublicationTimeSp {
+        fn from_args(a: &mut ArgumentStream) -> Result<IgnoredPublicationTimeSp, AE> {
+            let mut next_arg = || a.next().ok_or(AE::Missing);
+            let _: &str = next_arg()?;
+            let _: &str = next_arg()?;
+            Ok(IgnoredPublicationTimeSp)
         }
     }
 }
@@ -1110,7 +1239,7 @@ impl Signature {
     /// Return true if this signature has the identity key and signing key
     /// that match a given cert.
     fn matches_cert(&self, cert: &AuthCert) -> bool {
-        cert.key_ids() == &self.key_ids
+        cert.key_ids() == self.key_ids
     }
 
     /// If possible, find the right certificate for checking this signature
@@ -1186,7 +1315,7 @@ impl SignatureGroup {
     /// authorities.  This API requires that `n_authorities` is the number of
     /// authorities we believe in, and that every cert in `certs` belongs
     /// to a real authority.
-    fn validate(&self, n_authorities: u16, certs: &[AuthCert]) -> bool {
+    fn validate(&self, n_authorities: usize, certs: &[AuthCert]) -> bool {
         // A set of the authorities (by identity) who have have signed
         // this document.  We use a set here in case `certs` has more
         // than one certificate for a single authority.
@@ -1221,7 +1350,7 @@ impl SignatureGroup {
             }
         }
 
-        ok.len() > (n_authorities / 2) as usize
+        ok.len() > (n_authorities / 2)
     }
 }
 
@@ -1236,12 +1365,17 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use hex_literal::hex;
+    #[cfg(all(feature = "ns-vote", feature = "parse2"))]
+    use {
+        crate::parse2::{NetdocSigned as _, ParseInput, parse_netdoc},
+        std::fs,
+    };
 
     const CERTS: &str = include_str!("../../testdata/authcerts2.txt");
     const CONSENSUS: &str = include_str!("../../testdata/mdconsensus1.txt");
@@ -1271,7 +1405,7 @@ mod test {
             let cert = cert?.check_signature()?.dangerously_assume_timely();
             certs.push(cert);
         }
-        let auth_ids: Vec<_> = certs.iter().map(|c| &c.key_ids().id_fingerprint).collect();
+        let auth_ids: Vec<_> = certs.iter().map(|c| c.id_fingerprint()).collect();
 
         assert_eq!(certs.len(), 3);
 
@@ -1322,8 +1456,8 @@ mod test {
         assert!(!pv.supports_subver("HSDir", 3));
         let ip4 = "127.0.0.1:5002".parse::<SocketAddr>().unwrap();
         let ip6 = "[::1]:5002".parse::<SocketAddr>().unwrap();
-        assert!(r0.orport_addrs().any(|a| a == &ip4));
-        assert!(r0.orport_addrs().any(|a| a == &ip6));
+        assert!(r0.addrs().any(|a| a == ip4));
+        assert!(r0.addrs().any(|a| a == ip6));
 
         Ok(())
     }
@@ -1337,7 +1471,7 @@ mod test {
             let cert = cert?.check_signature()?.dangerously_assume_timely();
             certs.push(cert);
         }
-        let auth_ids: Vec<_> = certs.iter().map(|c| &c.key_ids().id_fingerprint).collect();
+        let auth_ids: Vec<_> = certs.iter().map(|c| c.id_fingerprint()).collect();
         assert_eq!(certs.len(), 4);
 
         let (_, _, consensus) = PlainConsensus::parse(PLAIN_CONSENSUS)?;
@@ -1354,6 +1488,24 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    #[cfg(all(feature = "ns-vote", feature = "parse2"))]
+    fn parse2_vote() -> anyhow::Result<()> {
+        let file = "testdata2/v3-status-votes--1";
+        let text = fs::read_to_string(file)?;
+
+        // TODO replace the poc struct here when we have parsing of proper whole votes
+        use crate::parse2::poc::netstatus::NetworkStatusSignedVote;
+
+        let input = ParseInput::new(&text, file);
+        let doc: NetworkStatusSignedVote = parse_netdoc(&input)?;
+
+        println!("{doc:?}");
+        println!("{:#?}", doc.inspect_unverified().0.r[0]);
+
+        Ok(())
+    }
+
     #[test]
     fn test_bad() {
         use crate::Pos;
@@ -1479,7 +1631,7 @@ mod test {
         assert_eq!(sr2.n_reveals, sr.n_reveals);
         assert_eq!(sr2.value.0, sr.value.0);
         assert_eq!(
-            sr2.timestamp.unwrap(),
+            sr2.timestamp.unwrap().0,
             humantime::parse_rfc3339("2022-01-20T12:34:56Z").unwrap()
         );
 
