@@ -106,7 +106,8 @@ pub use keyword::KeywordRef;
 pub use lex::{ArgumentStream, ItemStream, NoFurtherArguments, UnparsedItem, UnparsedObject};
 pub use lines::{Lines, Peeked, StrExt};
 pub use signatures::{
-    SignatureHashInputs, SignatureItemParseable, check_validity_time, sig_hash_methods,
+    SignatureHashInputs, SignatureItemParseable, check_validity_time,
+    check_validity_time_tolerance, sig_hash_methods,
 };
 pub use structural::{StopAt, StopPredicate};
 pub use traits::{
@@ -117,6 +118,8 @@ pub use traits::{
 #[doc(hidden)]
 pub use derive::netdoc_parseable_derive_debug;
 
+pub(crate) use internal_prelude::EP;
+
 //---------- input ----------
 
 /// Options for parsing
@@ -201,7 +204,7 @@ pub fn parse_netdoc<D: NetdocParseable>(input: &ParseInput<'_>) -> Result<D, Par
     })
 }
 
-/// Parse a network document - **toplevel entrypoint**
+/// Parse multiple concatenated network documents - **toplevel entrypoint**
 pub fn parse_netdoc_multiple<D: NetdocParseable>(
     input: &ParseInput<'_>,
 ) -> Result<Vec<D>, ParseError> {
@@ -214,3 +217,29 @@ pub fn parse_netdoc_multiple<D: NetdocParseable>(
         Ok(docs)
     })
 }
+
+/// Parse multiple network documents, also returning their offsets  - **toplevel entrypoint**
+///
+/// Each returned document is accompanied by the byte offsets of its start and end.
+///
+/// (The netdoc metaformat does not allow anything in between subsequent documents in a file,
+/// so the end of one document is the start of the next.)
+//
+// This returns byte offsets rather than string slices,
+// because the caller can always convert the offsets into string slices,
+// but it is not straightforward to convert string slices borrowed from some input string
+// into offsets, in a way that is obviously correct without nightly `str::substr_range`.
+pub fn parse_netdoc_multiple_with_offsets<D: NetdocParseable>(
+    input: &ParseInput<'_>,
+) -> Result<Vec<(D, usize, usize)>, ParseError> {
+    parse_internal::<_, D>(input, |items| {
+        let mut docs = vec![];
+        while items.peek_keyword()?.is_some() {
+            let start_pos = items.byte_position();
+            let doc = D::from_items(items, StopAt(false))?;
+            let end_pos = items.byte_position();
+            docs.push((doc, start_pos, end_pos));
+        }
+        Ok(docs)
+    })
+}
