@@ -6,21 +6,54 @@
 //! These types shouldn't be exposed outside of the netdoc crate.
 
 pub(crate) use b16impl::*;
-pub(crate) use b64impl::*;
+pub use b64impl::*;
 pub(crate) use curve25519impl::*;
 pub(crate) use ed25519impl::*;
 #[cfg(any(feature = "routerdesc", feature = "hs-common"))]
 pub(crate) use edcert::*;
 pub(crate) use fingerprint::*;
 pub(crate) use rsa::*;
-pub(crate) use timeimpl::*;
+pub use timeimpl::*;
 
+#[cfg(feature = "encode")]
+use {
+    crate::encode::{
+        self,
+        ItemEncoder,
+        ItemObjectEncodable,
+        ItemValueEncodable,
+        // `E` for "encode`; different from `parse2::MultiplicitySelector`
+        MultiplicitySelector as EMultiplicitySelector,
+    },
+    std::iter,
+};
 #[cfg(feature = "parse2")]
-use derive_deftly::Deftly;
+use {
+    crate::parse2::multiplicity::{
+        ItemSetMethods,
+        // `P2` for "parse2`; different from `encode::MultiplicitySelector`
+        MultiplicitySelector as P2MultiplicitySelector,
+        ObjectSetMethods,
+    },
+    crate::parse2::{ArgumentError, ArgumentStream, ItemArgumentParseable, ItemObjectParseable}, //
+};
 
 pub use nickname::Nickname;
 
-pub use fingerprint::Fingerprint;
+pub use fingerprint::{Base64Fingerprint, Fingerprint};
+
+pub use identified_digest::{DigestName, IdentifiedDigest};
+
+pub use ignored_impl::{Ignored, IgnoredItemOrObjectValue, NotPresent};
+
+use crate::NormalItemArgument;
+use derive_deftly::{Deftly, define_derive_deftly};
+use std::cmp::{self, PartialOrd};
+use std::fmt::{self, Display};
+use std::marker::PhantomData;
+use std::str::FromStr;
+use tor_error::{Bug, ErrorReport as _, internal};
+use void::{ResultVoidExt as _, Void};
 
 /// Describes a value that van be decoded from a bunch of bytes.
 ///
@@ -39,10 +72,31 @@ pub(crate) trait FromBytes: Sized {
 mod b64impl {
     use crate::{Error, NetdocErrorKind as EK, Pos, Result};
     use base64ct::{Base64, Base64Unpadded, Encoding};
+    use std::fmt::{self, Display};
     use std::ops::RangeBounds;
+    use subtle::{Choice, ConstantTimeEq};
 
     /// A byte array, encoded in base64 with optional padding.
-    pub(crate) struct B64(Vec<u8>);
+    ///
+    /// On output (`Display`), output is unpadded.
+    #[derive(Clone)]
+    #[allow(clippy::derived_hash_with_manual_eq)]
+    #[derive(Hash, derive_more::Debug, derive_more::From, derive_more::Into)]
+    #[debug(r#"B64("{self}")"#)]
+    pub struct B64(Vec<u8>);
+
+    impl ConstantTimeEq for B64 {
+        fn ct_eq(&self, other: &B64) -> Choice {
+            self.0.ct_eq(&other.0)
+        }
+    }
+    /// `B64` is `Eq` via its constant-time implementation.
+    impl PartialEq for B64 {
+        fn eq(&self, other: &B64) -> bool {
+            self.ct_eq(other).into()
+        }
+    }
+    impl Eq for B64 {}
 
     impl std::str::FromStr for B64 {
         type Err = Error;
@@ -60,9 +114,15 @@ mod b64impl {
         }
     }
 
+    impl Display for B64 {
+        fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            Display::fmt(&Base64Unpadded::encode_string(&self.0), f)
+        }
+    }
+
     impl B64 {
         /// Return the byte array from this object.
-        pub(crate) fn as_bytes(&self) -> &[u8] {
+        pub fn as_bytes(&self) -> &[u8] {
             &self.0[..]
         }
         /// Return this object if its length is within the provided bounds
@@ -78,18 +138,12 @@ mod b64impl {
         /// Try to convert this object into an array of N bytes.
         ///
         /// Return an error if the length is wrong.
-        pub(crate) fn into_array<const N: usize>(self) -> Result<[u8; N]> {
+        pub fn into_array<const N: usize>(self) -> Result<[u8; N]> {
             self.0
                 .try_into()
                 .map_err(|_| EK::BadObjectVal.with_msg("Invalid length on base64 data"))
         }
     }
-
-    impl From<B64> for Vec<u8> {
-        fn from(w: B64) -> Vec<u8> {
-            w.0
-        }
-    }
 }
 
 // ============================================================
@@ -198,19 +252,315 @@ mod ed25519impl {
 
 // ============================================================
 
-/// Ignored part of a network document.
+/// Dummy types like [`Ignored`]
+mod ignored_impl {
+    use super::*;
+
+    #[cfg(feature = "parse2")]
+    use crate::parse2::ErrorProblem as EP;
+
+    /// Part of a network document, that isn't actually there.
+    ///
+    /// Used as a standin in `ns_type!` calls in various netstatus `each_variety.rs`.
+    /// The effect is as if the field were omitted from the containing type.
+    ///
+    ///  * When used as item(s) (ie, a field type when deriving `NetdocParseable\[Fields\]`):
+    ///    **ignores any number** of items with that field's keyword during parsing,
+    ///    and emits none during encoding.
+    ///
+    ///    (To *reject* documents containing this item, use `Option<Void>`,
+    ///    but note that the spec says unknown items should be ignored,
+    ///    which would normally include items which are merely missing from one variety.)
+    ///
+    ///  * When used as an argument (ie, a field type when deriving `ItemValueParseable`,
+    ///    or with `netdoc(single_arg)`  when deriving `NetdocParseable\[Fields\]`):
+    ///    consumes **no arguments** during parsing, and emits none during encoding.
+    ///
+    ///  * When used as an object field (ie, `netdoc(object)` when deriving `ItemValueParseable`):
+    ///    **rejects** an object - failing the parse if one is present.
+    ///    (Functions similarly to `Option<Void>`, but prefer `NotPresent` as it's clearer.)
+    ///
+    /// There are bespoke impls of the multiplicity traits
+    /// `ItemSetMethods` and `ObjectSetMethods`:
+    /// don't wrap this type in `Option` or `Vec`.
+    //
+    // TODO we'll need to implement ItemArgument etc., for encoding, too.
+    #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd, Default)]
+    #[allow(clippy::exhaustive_structs)]
+    #[cfg_attr(
+        feature = "parse2",
+        derive(Deftly),
+        derive_deftly(NetdocParseableFields)
+    )]
+    pub struct NotPresent;
+
+    /// Ignored part of a network document.
+    ///
+    /// With `parse2`, can be used as an item, object, or even flattened-fields.
+    ///
+    /// When deriving `parse2` traits, and a field is absent in a particular netstatus variety,
+    /// use `ns_type!` with [`NotPresent`], rather than `Ignored`.
+    ///
+    /// During encoding as an Items or Objects, will be entirely omitted,
+    /// via the multiplicity arrangements.
+    ///
+    /// Cannot be encoded as an Argument: if this is not the last
+    /// Argument, we need something to put into the output document to avoid generating
+    /// a document with the arguments out of step.  If it *is* the last argument,
+    /// it could simply be omitted, since additional arguments are in any case ignored.
+    #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd, Default)]
+    #[cfg_attr(
+        feature = "parse2",
+        derive(Deftly),
+        derive_deftly(ItemValueParseable, NetdocParseableFields)
+    )]
+    #[allow(clippy::exhaustive_structs)]
+    pub struct Ignored;
+
+    /// An Item or Object that would be ignored during parsing and is omitted during encoding
+    ///
+    /// This is the "single" item type for encoding multiplicity for Items or Objects,
+    /// for [`Ignored`].
+    ///
+    /// This type is uninhabited.
+    pub struct IgnoredItemOrObjectValue(Void);
+
+    #[cfg(feature = "parse2")]
+    impl ItemSetMethods for P2MultiplicitySelector<NotPresent> {
+        type Each = Ignored;
+        type Field = NotPresent;
+        fn can_accumulate(self, _acc: &Option<NotPresent>) -> Result<(), EP> {
+            Ok(())
+        }
+        fn accumulate(self, _acc: &mut Option<NotPresent>, _item: Ignored) -> Result<(), EP> {
+            Ok(())
+        }
+        fn finish(self, _acc: Option<NotPresent>, _: &'static str) -> Result<NotPresent, EP> {
+            Ok(NotPresent)
+        }
+    }
+
+    #[cfg(feature = "parse2")]
+    impl ItemArgumentParseable for NotPresent {
+        fn from_args(_: &mut ArgumentStream) -> Result<NotPresent, ArgumentError> {
+            Ok(NotPresent)
+        }
+    }
+
+    #[cfg(feature = "parse2")]
+    impl ObjectSetMethods for P2MultiplicitySelector<NotPresent> {
+        type Field = NotPresent;
+        type Each = Void;
+        fn resolve_option(self, _found: Option<Void>) -> Result<NotPresent, EP> {
+            Ok(NotPresent)
+        }
+    }
+
+    #[cfg(feature = "encode")]
+    impl<'f> encode::MultiplicityMethods<'f> for EMultiplicitySelector<NotPresent> {
+        type Field = NotPresent;
+        type Each = Void;
+        fn iter_ordered(self, _: &'f Self::Field) -> impl Iterator<Item = &'f Self::Each> {
+            iter::empty()
+        }
+    }
+
+    #[cfg(feature = "encode")]
+    impl encode::OptionalityMethods for EMultiplicitySelector<NotPresent> {
+        type Field = NotPresent;
+        type Each = Void;
+        fn as_option<'f>(self, _: &'f Self::Field) -> Option<&'f Self::Each> {
+            None
+        }
+    }
+
+    impl FromStr for Ignored {
+        type Err = Void;
+        fn from_str(_s: &str) -> Result<Ignored, Void> {
+            Ok(Ignored)
+        }
+    }
+
+    #[cfg(feature = "parse2")]
+    impl ItemArgumentParseable for Ignored {
+        fn from_args(_: &mut ArgumentStream) -> Result<Ignored, ArgumentError> {
+            Ok(Ignored)
+        }
+    }
+
+    #[cfg(feature = "parse2")]
+    impl ItemObjectParseable for Ignored {
+        fn check_label(_label: &str) -> Result<(), EP> {
+            // allow any label
+            Ok(())
+        }
+        fn from_bytes(_input: &[u8]) -> Result<Self, EP> {
+            Ok(Ignored)
+        }
+    }
+
+    #[cfg(feature = "parse2")]
+    impl ObjectSetMethods for P2MultiplicitySelector<Ignored> {
+        type Field = Ignored;
+        type Each = Ignored;
+        fn resolve_option(self, _found: Option<Ignored>) -> Result<Ignored, EP> {
+            Ok(Ignored)
+        }
+    }
+
+    #[cfg(feature = "encode")]
+    impl<'f> encode::MultiplicityMethods<'f> for EMultiplicitySelector<Ignored> {
+        type Field = Ignored;
+        type Each = IgnoredItemOrObjectValue;
+        fn iter_ordered(self, _: &'f Self::Field) -> impl Iterator<Item = &'f Self::Each> {
+            iter::empty()
+        }
+    }
+
+    #[cfg(feature = "encode")]
+    impl encode::OptionalityMethods for EMultiplicitySelector<Ignored> {
+        type Field = Ignored;
+        type Each = IgnoredItemOrObjectValue;
+        fn as_option<'f>(self, _: &'f Self::Field) -> Option<&'f Self::Each> {
+            None
+        }
+    }
+
+    #[cfg(feature = "encode")]
+    impl ItemValueEncodable for IgnoredItemOrObjectValue {
+        fn write_item_value_onto(&self, _: ItemEncoder) -> Result<(), Bug> {
+            void::unreachable(self.0)
+        }
+    }
+
+    #[cfg(feature = "encode")]
+    impl ItemObjectEncodable for IgnoredItemOrObjectValue {
+        fn label(&self) -> &str {
+            void::unreachable(self.0)
+        }
+        fn write_object_onto(&self, _: &mut Vec<u8>) -> Result<(), Bug> {
+            void::unreachable(self.0)
+        }
+    }
+}
+
+// ============================================================
+
+/// Information about unknown values, which may have been retained as a `T`
 ///
-/// With `parse2`, can be used as an item, argument, object, or even flattened-fields.
+/// Won't grow additional variants - but, `Retained` is only included conditionally.
 ///
-/// If an optional item, argument or object is wanted, use `Option<Ignore>`.
-#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd)]
-#[cfg_attr(
-    feature = "parse2",
-    derive(Deftly),
-    derive_deftly(ItemValueParseable, NetdocParseableFields)
-)]
-#[allow(clippy::exhaustive_structs)]
-pub struct Ignored;
+/// Also used in the form `Unknown<()>` to indicate whether unknown values *should* be retained.
+///
+/// ### Example
+///
+/// ```
+/// # {
+/// #![cfg(feature = "retain-unknown")]
+///
+/// use tor_netdoc::types::Unknown;
+///
+/// let mut unk: Unknown<Vec<String>> = Unknown::new_retained_default();
+/// unk.with_mut_unknown(|u| u.push("something-we-found".into()));
+/// assert_eq!(unk.into_retained().unwrap(), ["something-we-found"]);
+/// # }
+/// ```
+///
+/// ### Equality comparison, semantics
+///
+/// Two `Unknown` are consider equal if both have the same record of unknown values,
+/// or if neither records unknown values at all.
+///
+/// `Unknown` is not `Eq` or `Ord` because we won't want to relate a `Discarded`
+/// to a `Retained`.  That would be a a logic error.  `partial_cmp` gives `None` for this.
+#[derive(Debug, PartialEq, Clone, Copy, Hash)]
+#[non_exhaustive]
+pub enum Unknown<T> {
+    /// The parsing discarded unknown values and they are no longer available.
+    Discarded(PhantomData<T>),
+
+    /// The document parsing retained (or should retain) unknown values.
+    #[cfg(feature = "retain-unknown")]
+    Retained(T),
+}
+
+impl<T> Unknown<T> {
+    /// Create an `Unknown` which specifies that values were discarded (or should be)
+    pub fn new_discard() -> Self {
+        Unknown::Discarded(PhantomData)
+    }
+
+    /// Map the `Retained`, if there is one
+    pub fn map<U>(self, f: impl FnOnce(T) -> U) -> Unknown<U> {
+        self.try_map(move |t| Ok::<_, Void>(f(t))).void_unwrap()
+    }
+
+    /// Map the `Retained`, fallibly
+    pub fn try_map<U, E>(self, f: impl FnOnce(T) -> Result<U, E>) -> Result<Unknown<U>, E> {
+        Ok(match self {
+            Unknown::Discarded(_) => Unknown::Discarded(PhantomData),
+            #[cfg(feature = "retain-unknown")]
+            Unknown::Retained(t) => Unknown::Retained(f(t)?),
+        })
+    }
+
+    /// Obtain an `Unknown` containing (maybe) a reference
+    pub fn as_ref(&self) -> Option<&T> {
+        match self {
+            Unknown::Discarded(_) => None,
+            #[cfg(feature = "retain-unknown")]
+            Unknown::Retained(t) => Some(t),
+        }
+    }
+
+    /// Obtain the `Retained` data
+    ///
+    /// Treats lack of retention as an internal error.
+    #[cfg(feature = "retain-unknown")]
+    pub fn into_retained(self) -> Result<T, Bug> {
+        match self {
+            Unknown::Discarded(_) => Err(internal!("Unknown::retained but data not collected")),
+            Unknown::Retained(t) => Ok(t),
+        }
+    }
+
+    /// Start recording unknown information, with a default value for `T`
+    #[cfg(feature = "retain-unknown")]
+    pub fn new_retained_default() -> Self
+    where
+        T: Default,
+    {
+        Unknown::Retained(T::default())
+    }
+
+    /// Update the `Retained`, if there is one
+    ///
+    /// Intended for use in parsing, when we encounter an unknown value.
+    ///
+    /// Not provided in `try_` form.  If you think you need this, instead, unconditionally
+    /// parse and verify the unknown value, and then conditionally insert it with this function.
+    /// Don't parse it conditionally - that would skip some validation.
+    pub fn with_mut_unknown(&mut self, f: impl FnOnce(&mut T)) {
+        match self {
+            Unknown::Discarded(_) => {}
+            #[cfg(feature = "retain-unknown")]
+            Unknown::Retained(t) => f(t),
+        }
+    }
+}
+
+impl<T: PartialOrd> PartialOrd for Unknown<T> {
+    fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
+        use Unknown::*;
+        match (self, other) {
+            (Discarded(_), Discarded(_)) => Some(cmp::Ordering::Equal),
+            #[cfg(feature = "retain-unknown")]
+            (Discarded(_), Retained(_)) | (Retained(_), Discarded(_)) => None,
+            #[cfg(feature = "retain-unknown")]
+            (Retained(a), Retained(b)) => a.partial_cmp(b),
+        }
+    }
+}
 
 // ============================================================
 
@@ -227,8 +577,10 @@ mod timeimpl {
     /// space between the date and time.
     ///
     /// (Example: "2020-10-09 17:38:12")
-    #[derive(derive_more::Into, derive_more::From)]
-    pub(crate) struct Iso8601TimeSp(SystemTime);
+    #[derive(Debug, Copy, Clone, Ord, PartialOrd, Eq, PartialEq, Hash)] //
+    #[derive(derive_more::Into, derive_more::From, derive_more::Deref)]
+    #[allow(clippy::exhaustive_structs)]
+    pub struct Iso8601TimeSp(pub SystemTime);
 
     /// Formatting object for parsing the space-separated Iso8601 format.
     const ISO_8601SP_FMT: &[FormatItem] =
@@ -274,8 +626,10 @@ mod timeimpl {
     /// The timezone is not included in the string representation; `+0000` is implicit.
     ///
     /// (Example: "2020-10-09T17:38:12")
-    #[derive(derive_more::Into, derive_more::From)]
-    pub(crate) struct Iso8601TimeNoSp(SystemTime);
+    #[derive(Debug, Copy, Clone, Ord, PartialOrd, Eq, PartialEq, Hash)] //
+    #[derive(derive_more::Into, derive_more::From, derive_more::Deref)]
+    #[allow(clippy::exhaustive_structs)]
+    pub struct Iso8601TimeNoSp(pub SystemTime);
 
     /// Formatting object for parsing the space-separated Iso8601 format.
     const ISO_8601NOSP_FMT: &[FormatItem] =
@@ -298,6 +652,8 @@ mod timeimpl {
             write!(f, "{}", fmt_with(self.0, ISO_8601NOSP_FMT)?)
         }
     }
+
+    impl crate::NormalItemArgument for Iso8601TimeNoSp {}
 }
 
 /// Types for decoding RSA keys
@@ -306,24 +662,37 @@ mod rsa {
     use std::ops::RangeBounds;
     use tor_llcrypto::pk::rsa::PublicKey;
 
-    /// An RSA public key, as parsed from a base64-encoded object.
+    /// The fixed exponent which we require when parsing any RSA key in a netdoc
+    //
+    // TODO this value is duplicated a lot in the v1 parser
+    pub(crate) const RSA_FIXED_EXPONENT: u32 = 65537;
+
+    /// The fixed exponent which we require when parsing any RSA key in a netdoc
+    //
+    // TODO this value is duplicated a lot in the v1 parser
+    pub(crate) const RSA_MIN_BITS: usize = 1024;
+
+    /// RSA public key, partially processed by `crate::paarse`.
+    ///
+    /// As parsed from a base64-encoded object.
+    /// They key's properties (exponent and size) haven't been checked.
     #[allow(non_camel_case_types)]
     #[derive(Clone, Debug)]
-    pub(crate) struct RsaPublic(PublicKey, Pos);
+    pub(crate) struct RsaPublicParse1Helper(PublicKey, Pos);
 
-    impl From<RsaPublic> for PublicKey {
-        fn from(k: RsaPublic) -> PublicKey {
+    impl From<RsaPublicParse1Helper> for PublicKey {
+        fn from(k: RsaPublicParse1Helper) -> PublicKey {
             k.0
         }
     }
-    impl super::FromBytes for RsaPublic {
+    impl super::FromBytes for RsaPublicParse1Helper {
         fn from_bytes(b: &[u8], pos: Pos) -> Result<Self> {
             let key = PublicKey::from_der(b)
                 .ok_or_else(|| EK::BadObjectVal.with_msg("unable to decode RSA public key"))?;
-            Ok(RsaPublic(key, pos))
+            Ok(RsaPublicParse1Helper(key, pos))
         }
     }
-    impl RsaPublic {
+    impl RsaPublicParse1Helper {
         /// Give an error if the exponent of this key is not 'e'
         pub(crate) fn check_exponent(self, e: u32) -> Result<Self> {
             if self.0.exponent_is(e) {
@@ -410,50 +779,173 @@ mod edcert {
     }
 }
 
+/// Digest identifeirs, and digests in the form `ALGORITHM=BASE64U`
+///
+/// As found in a vote's `m` line.
+mod identified_digest {
+    use super::*;
+
+    define_derive_deftly! {
+        /// impl `FromStr` and `Display` for an enum with unit variants but also "unknown"
+        ///
+        /// Expected input: an enum whose variants are either
+        ///  * unit variants, perhaps with `#[deftly(string_repr = "string")]`
+        ///  * singleton tuple variant, containing `String` (or near equivalent)
+        ///
+        /// If `#[deftly(string_repro)]` is not specified,
+        /// the default is snake case of the variant name.
+        //
+        // This macro may seem overkill, but open-coding these impls gives opportunities
+        // for mismatches between FromStr, Display, and the variant name.
+        //
+        // TODO consider putting this in tor-basic-utils (maybe with a better name),
+        // or possibly asking if derive_more want their FromStr to have this.
+        StringReprUnitsOrUnknown for enum, expect items, beta_deftly:
+
+        ${define STRING_REPR {
+            ${vmeta(string_repr)
+              as str,
+              default { ${concat ${snake_case $vname}} }
+            }
+        }}
+
+        impl FromStr for $ttype {
+            type Err = Void;
+            fn from_str(s: &str) -> Result<Self, Void> {
+                $(
+                    ${when v_is_unit}
+                    if s == $STRING_REPR {
+                        return Ok($vtype)
+                    }
+                )
+                $(
+                    ${when not(v_is_unit)} // anything else had better be Unknown
+                    // not using `return ..;` makes this a syntax error if there are several.
+                    Ok($vtype { 0: s.into() })
+                )
+            }
+        }
+        impl Display for $ttype {
+            fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+                let s: &str = match self {
+                    $(
+                        ${when v_is_unit}
+                        $vtype => $STRING_REPR,
+                    )
+                    $(
+                        ${when not(v_is_unit)}
+                        $vpat => f_0,
+                    )
+                };
+                Display::fmt(s, f)
+            }
+        }
+    }
+
+    /// The name of a digest algorithm.
+    ///
+    /// Can represent an unrecognised algorithm, so it's parsed and reproduced.
+    #[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Deftly)]
+    #[derive_deftly(StringReprUnitsOrUnknown)]
+    #[non_exhaustive]
+    pub enum DigestName {
+        /// SHA-256
+        Sha256,
+        /// Unknown
+        Unknown(String),
+    }
+
+    /// A single digest made with a nominated digest algorithm, `ALGORITHM=DIGEST`
+    #[derive(Debug, Clone, Eq, PartialEq, Hash, derive_more::Display)]
+    #[display("{alg}={value}")]
+    #[non_exhaustive]
+    pub struct IdentifiedDigest {
+        /// The algorithm name.
+        alg: DigestName,
+
+        /// The digest value.
+        ///
+        /// Invariant: length is correct for `alg`, assuming `alg` is known.
+        value: B64,
+    }
+
+    impl NormalItemArgument for DigestName {}
+    impl NormalItemArgument for IdentifiedDigest {}
+
+    /// Invalid syntax parsing an `IdentifiedDigest`
+    #[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, thiserror::Error)]
+    #[error("invalid syntax, espected ALGORITHM=DIGEST: {0}")]
+    pub struct IdentifiedDigestParseError(String);
+
+    impl FromStr for IdentifiedDigest {
+        type Err = IdentifiedDigestParseError;
+
+        fn from_str(s: &str) -> Result<Self, Self::Err> {
+            (|| {
+                let (alg, value) = s.split_once('=').ok_or("missing equals sign")?;
+
+                let alg = alg.parse().void_unwrap();
+                let value = value
+                    .parse::<B64>()
+                    .map_err(|e| format!("bad value: {}", e.report()))?;
+
+                if let Some(exp_len) = (|| {
+                    Some({
+                        use DigestName::*;
+                        match alg {
+                            Sha256 => 32,
+                            Unknown(_) => None?,
+                        }
+                    })
+                })() {
+                    let val_len = value.as_bytes().len();
+                    if val_len != exp_len {
+                        return Err(format!("got {val_len} bytes, expected {exp_len}"));
+                    }
+                }
+
+                Ok(IdentifiedDigest { alg, value })
+            })()
+            .map_err(IdentifiedDigestParseError)
+        }
+    }
+}
+
 /// Types for decoding RSA fingerprints
 mod fingerprint {
     use crate::{Error, NetdocErrorKind as EK, Pos, Result};
+    use base64ct::{Base64Unpadded, Encoding as _};
+    use std::fmt::{self, Display};
     use tor_llcrypto::pk::rsa::RsaIdentity;
 
     /// A hex-encoded RSA key identity (fingerprint) with spaces in it.
     ///
     /// Netdoc parsing adapter for [`RsaIdentity`]
-    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref)]
+    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref, derive_more::Into)]
     #[allow(clippy::exhaustive_structs)]
     pub(crate) struct SpFingerprint(pub RsaIdentity);
 
     /// A hex-encoded fingerprint with no spaces.
     ///
     /// Netdoc parsing adapter for [`RsaIdentity`]
-    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref)]
+    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref, derive_more::Into)]
     #[allow(clippy::exhaustive_structs)]
     pub struct Fingerprint(pub RsaIdentity);
 
+    /// A base64-encoded fingerprint (unpadded)
+    ///
+    /// Netdoc parsing adapter for [`RsaIdentity`]
+    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref, derive_more::Into)]
+    #[allow(clippy::exhaustive_structs)]
+    pub struct Base64Fingerprint(pub RsaIdentity);
+
     /// A "long identity" in the format used for Family members.
     ///
     /// Netdoc parsing adapter for [`RsaIdentity`]
-    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref)]
+    #[derive(Debug, Clone, Eq, PartialEq, derive_more::Deref, derive_more::Into)]
     #[allow(clippy::exhaustive_structs)]
     pub(crate) struct LongIdent(pub RsaIdentity);
 
-    impl From<SpFingerprint> for RsaIdentity {
-        fn from(f: SpFingerprint) -> RsaIdentity {
-            f.0
-        }
-    }
-
-    impl From<LongIdent> for RsaIdentity {
-        fn from(f: LongIdent) -> RsaIdentity {
-            f.0
-        }
-    }
-
-    impl From<Fingerprint> for RsaIdentity {
-        fn from(f: Fingerprint) -> RsaIdentity {
-            f.0
-        }
-    }
-
     /// Helper: parse an identity from a hexadecimal string
     fn parse_hex_ident(s: &str) -> Result<RsaIdentity> {
         RsaIdentity::from_hex(s).ok_or_else(|| {
@@ -471,6 +963,25 @@ mod fingerprint {
         }
     }
 
+    impl std::str::FromStr for Base64Fingerprint {
+        type Err = Error;
+        fn from_str(s: &str) -> Result<Base64Fingerprint> {
+            let b = s.parse::<super::B64>()?;
+            let ident = RsaIdentity::from_bytes(b.as_bytes()).ok_or_else(|| {
+                EK::BadArgument
+                    .at_pos(Pos::at(s))
+                    .with_msg("Wrong identity length")
+            })?;
+            Ok(Base64Fingerprint(ident))
+        }
+    }
+
+    impl Display for Base64Fingerprint {
+        fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            Display::fmt(&Base64Unpadded::encode_string(self.as_bytes()), f)
+        }
+    }
+
     impl std::str::FromStr for Fingerprint {
         type Err = Error;
         fn from_str(s: &str) -> Result<Fingerprint> {
@@ -479,6 +990,12 @@ mod fingerprint {
         }
     }
 
+    impl Display for Fingerprint {
+        fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+            Display::fmt(&hex::encode_upper(self.as_bytes()), f)
+        }
+    }
+
     impl std::str::FromStr for LongIdent {
         type Err = Error;
         fn from_str(mut s: &str) -> Result<LongIdent> {
@@ -492,6 +1009,9 @@ mod fingerprint {
             Ok(LongIdent(ident))
         }
     }
+
+    impl crate::NormalItemArgument for Fingerprint {}
+    impl crate::NormalItemArgument for Base64Fingerprint {}
 }
 
 /// A type for relay nicknames
@@ -545,6 +1065,8 @@ mod nickname {
             }
         }
     }
+
+    impl crate::NormalItemArgument for Nickname {}
 }
 
 #[cfg(test)]
@@ -558,7 +1080,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
@@ -812,7 +1334,7 @@ mod test {
         FS3Dd/Dx/a6jAgMBAAE=
         "#;
         let key_bytes = base64_decode_ignore_ws(key_b64).unwrap();
-        let rsa = RsaPublic::from_vec(key_bytes, Pos::None).unwrap();
+        let rsa = RsaPublicParse1Helper::from_vec(key_bytes, Pos::None).unwrap();
 
         let bits = tor_llcrypto::pk::rsa::PublicKey::from(rsa.clone()).bits();
         assert_eq!(bits, 3072);
@@ -826,7 +1348,7 @@ mod test {
         assert!(rsa.check_len(4096..).is_err());
 
         // A string of bytes that is not an RSA key.
-        let failure = RsaPublic::from_vec(vec![1, 2, 3], Pos::None);
+        let failure = RsaPublicParse1Helper::from_vec(vec![1, 2, 3], Pos::None);
         assert!(failure.is_err());
     }
 
@@ -894,6 +1416,7 @@ mod test {
         assert_eq!(RsaIdentity::from(fp2.parse::<Fingerprint>()?), k);
         assert!(fp3.parse::<Fingerprint>().is_err());
         assert!(fp4.parse::<Fingerprint>().is_err());
+        assert_eq!(Fingerprint(k).to_string(), fp2);
 
         assert!(fp1.parse::<LongIdent>().is_err());
         assert_eq!(RsaIdentity::from(fp2.parse::<LongIdent>()?), k);
@@ -902,6 +1425,11 @@ mod test {
 
         assert!("xxxx".parse::<Fingerprint>().is_err());
         assert!("ffffffffff".parse::<Fingerprint>().is_err());
+
+        let fp_b64 = "dGepfRnNK08rwDiKqZxeZ3EPhH4";
+        assert_eq!(RsaIdentity::from(fp_b64.parse::<Base64Fingerprint>()?), k);
+        assert_eq!(Base64Fingerprint(k).to_string(), fp_b64);
+
         Ok(())
     }
 
