@@ -22,37 +22,28 @@ mod conflux;
 mod control;
 pub(super) mod syncview;
 
-use crate::circuit::UniqId;
-use crate::circuit::celltypes::ClientCircChanMsg;
 use crate::circuit::circhop::SendRelayCell;
+use crate::circuit::{CircuitRxReceiver, UniqId};
 use crate::client::circuit::padding::{PaddingController, PaddingEvent, PaddingEventStream};
-use crate::client::circuit::{CircuitRxReceiver, TimeoutEstimator};
-#[cfg(feature = "hs-service")]
-use crate::client::stream::{IncomingStreamRequest, IncomingStreamRequestFilter};
+use crate::client::circuit::{ClientCircChanMsg, TimeoutEstimator};
 use crate::client::{HopLocation, TargetHop};
 use crate::crypto::cell::HopNum;
 use crate::crypto::handshake::ntor_v3::NtorV3PublicKey;
-use crate::memquota::{CircuitAccount, StreamAccount};
+use crate::memquota::CircuitAccount;
+use crate::stream::CloseStreamBehavior;
 use crate::stream::cmdcheck::AnyCmdChecker;
-use crate::stream::flow_ctrl::state::StreamRateLimit;
-use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
-use crate::stream::queue::StreamQueueReceiver;
-use crate::stream::{CloseStreamBehavior, StreamMpscSender};
 use crate::streammap;
 use crate::tunnel::{TunnelId, TunnelScopedCircId};
 use crate::util::err::ReactorError;
-use crate::util::notify::NotifyReceiver;
 use crate::util::skew::ClockSkew;
 use crate::{Error, Result};
 use circuit::Circuit;
 use conflux::ConfluxSet;
 use control::ControlHandler;
-use postage::watch;
 use std::cmp::Ordering;
 use std::collections::BinaryHeap;
-use std::mem::size_of;
 use tor_cell::relaycell::flow_ctrl::XonKbpsEwma;
-use tor_cell::relaycell::msg::{AnyRelayMsg, Sendme};
+use tor_cell::relaycell::msg::Sendme;
 use tor_cell::relaycell::{AnyRelayMsgOuter, RelayCellFormat, StreamId, UnparsedRelayMsg};
 use tor_error::{Bug, bad_api_usage, debug_report, internal, into_bad_api_usage};
 use tor_rtcompat::{DynTimeProvider, SleepProvider};
@@ -71,17 +62,18 @@ use crate::channel::Channel;
 use crate::conflux::msghandler::RemoveLegReason;
 use crate::crypto::handshake::ntor::{NtorClient, NtorPublicKey};
 use circuit::CircuitCmd;
-use derive_deftly::Deftly;
 use derive_more::From;
 use smallvec::smallvec;
 use tor_cell::chancell::CircId;
 use tor_llcrypto::pk;
-use tor_memquota::derive_deftly_template_HasMemoryCost;
 use tor_memquota::mq_queue::{self, MpscSpec};
 use tracing::{debug, info, instrument, trace, warn};
 
 use super::circuit::{MutableState, TunnelMutableState};
 
+#[cfg(feature = "hs-service")]
+use {crate::client::stream::IncomingStreamRequestFilter, crate::stream::incoming::StreamReqInfo};
+
 #[cfg(feature = "conflux")]
 use {
     crate::conflux::msghandler::{ConfluxCmd, OooRelayMsg},
@@ -105,8 +97,6 @@ pub(super) type ConfluxHandshakeResult = Vec<StdResult<(), ConfluxHandshakeError
 #[cfg(feature = "conflux")]
 pub(super) type ConfluxLinkResultChannel = ReactorResultChannel<ConfluxHandshakeResult>;
 
-pub(crate) use circuit::{RECV_WINDOW_INIT, STREAM_READER_BUFFER};
-
 /// MPSC queue containing stream requests
 #[cfg(feature = "hs-service")]
 type StreamReqSender = mq_queue::Sender<StreamReqInfo, MpscSpec>;
@@ -131,7 +121,7 @@ pub(crate) enum CircuitHandshake {
     },
 }
 
-// TODO: the RunOnceCmd/RunOnceCmdInner/CircuitCmd/CircuitAction enum
+// TODO: the RunOnceCmd/RunOnceCmdInner/CircuitCmd/CircuitEvent enum
 // proliferation is a bit bothersome, but unavoidable with the current design.
 //
 // We should consider getting rid of some of these enums (if possible),
@@ -285,13 +275,13 @@ enum RunOnceCmdInner {
         /// The out-of-order message.
         msg: OooRelayMsg,
     },
-    /// Take a padding-related action on a circuit leg.
+    /// Take a padding-related event on a circuit leg.
     #[cfg(feature = "circ-padding")]
     PaddingAction {
-        /// The leg to action on.
+        /// The leg to event on.
         leg: UniqId,
-        /// The action to take.
-        padding_action: PaddingEvent,
+        /// The event to take.
+        padding_event: PaddingEvent,
     },
     /// Perform a clean shutdown on this circuit.
     CleanShutdown,
@@ -340,7 +330,7 @@ impl RunOnceCmdInner {
 /// A command to execute at the end of [`Reactor::run_once`].
 #[derive(From, Debug)]
 #[allow(clippy::large_enum_variant)] // TODO #2003: should we resolve this?
-enum CircuitAction {
+enum CircuitEvent {
     /// Run a single `CircuitCmd` command.
     RunCmd {
         /// The unique identifier of the circuit leg to run the command on
@@ -363,7 +353,7 @@ enum CircuitAction {
     /// from the reactor's conflux set, without necessarily tearing down
     /// the whole set or shutting down the reactor.
     ///
-    /// Note: this action *can* cause the reactor to shut down
+    /// Note: this event *can* cause the reactor to shut down
     /// (and the conflux set to be closed).
     ///
     /// See the [`ConfluxSet::remove`] docs for more on the exact behavior of this command.
@@ -380,31 +370,40 @@ enum CircuitAction {
         /// handshake to indicate the reason the handshake failed.
         reason: RemoveLegReason,
     },
-    /// Take some action (blocking or unblocking a circuit, or sending padding)
+    /// Take some event (blocking or unblocking a circuit, or sending padding)
     /// based on the circuit padding backend code.
     PaddingAction {
-        /// The leg on which to take the padding action.
+        /// The leg on which to take the padding event .
         leg: UniqId,
-        /// The action to take.
-        padding_action: PaddingEvent,
+        /// The event to take.
+        padding_event: PaddingEvent,
+    },
+    /// Protocol violation. This leads for now to the close of the circuit reactor. The
+    /// error causing the violation is set in err.
+    ProtoViolation {
+        /// The error that causes this protocol violation.
+        err: crate::Error,
     },
 }
 
-impl CircuitAction {
-    /// Return the ordering with which we should handle this action
-    /// within a list of actions returned by a single call to next_circ_action().
+impl CircuitEvent {
+    /// Return the ordering with which we should handle this event
+    /// within a list of events returned by a single call to next_circ_event().
     ///
     /// NOTE: Please do not make this any more complicated:
     /// It is a consequence of a kludge that we need this sorting at all.
     /// Assuming that eventually, we switch away from the current
-    /// poll-oriented `next_circ_action` design,
+    /// poll-oriented `next_circ_event` design,
     /// we may be able to get rid of this entirely.
     fn order_within_batch(&self) -> u8 {
-        use CircuitAction as CA;
+        use CircuitEvent as CA;
         use PaddingEvent as PE;
-        const EARLY: u8 = 0;
-        const NORMAL: u8 = 1;
-        const LATE: u8 = 2;
+        // This immediate state MUST NOT be used for events emitting cells. At the moment, it is
+        // only used by the protocol violation event which leads to a shutdown of the reactor.
+        const IMMEDIATE: u8 = 0;
+        const EARLY: u8 = 1;
+        const NORMAL: u8 = 2;
+        const LATE: u8 = 3;
 
         // We use this ordering to move any "StartBlocking" to the _end_ of a batch and
         // "StopBlocking" to the start.
@@ -420,13 +419,14 @@ impl CircuitAction {
             CA::HandleCell { .. } => NORMAL,
             CA::RemoveLeg { .. } => NORMAL,
             #[cfg(feature = "circ-padding")]
-            CA::PaddingAction { padding_action, .. } => match padding_action {
+            CA::PaddingAction { padding_event, .. } => match padding_event {
                 PE::StopBlocking => EARLY,
                 PE::SendPadding(..) => NORMAL,
                 PE::StartBlocking(..) => LATE,
             },
             #[cfg(not(feature = "circ-padding"))]
             CA::PaddingAction { .. } => NORMAL,
+            CA::ProtoViolation { .. } => IMMEDIATE,
         }
     }
 }
@@ -625,46 +625,6 @@ struct CellHandlers {
     incoming_stream_req_handler: Option<IncomingStreamRequestHandler>,
 }
 
-/// Information about an incoming stream request.
-#[cfg(feature = "hs-service")]
-#[derive(Debug, Deftly)]
-#[derive_deftly(HasMemoryCost)]
-pub(crate) struct StreamReqInfo {
-    /// The [`IncomingStreamRequest`].
-    pub(crate) req: IncomingStreamRequest,
-    /// The ID of the stream being requested.
-    pub(crate) stream_id: StreamId,
-    /// The [`HopNum`].
-    //
-    // TODO: When we add support for exit relays, we need to turn this into an Option<HopNum>.
-    // (For outbound messages (towards relays), there is only one hop that can send them: the client.)
-    //
-    // TODO: For onion services, we might be able to enforce the HopNum earlier: we would never accept an
-    // incoming stream request from two separate hops.  (There is only one that's valid.)
-    pub(crate) hop: HopLocation,
-    /// The format which must be used with this stream to encode messages.
-    #[deftly(has_memory_cost(indirect_size = "0"))]
-    pub(crate) relay_cell_format: RelayCellFormat,
-    /// A channel for receiving messages from this stream.
-    #[deftly(has_memory_cost(indirect_size = "0"))] // estimate
-    pub(crate) receiver: StreamQueueReceiver,
-    /// A channel for sending messages to be sent on this stream.
-    #[deftly(has_memory_cost(indirect_size = "size_of::<AnyRelayMsg>()"))] // estimate
-    pub(crate) msg_tx: StreamMpscSender<AnyRelayMsg>,
-    /// A [`Stream`](futures::Stream) that provides updates to the rate limit for sending data.
-    // TODO(arti#2068): we should consider making this an `Option`
-    // the `watch::Sender` owns the indirect data
-    #[deftly(has_memory_cost(indirect_size = "0"))]
-    pub(crate) rate_limit_stream: watch::Receiver<StreamRateLimit>,
-    /// A [`Stream`](futures::Stream) that provides notifications when a new drain rate is
-    /// requested.
-    #[deftly(has_memory_cost(indirect_size = "0"))]
-    pub(crate) drain_rate_request_stream: NotifyReceiver<DrainRateRequest>,
-    /// The memory quota account to be used for this stream
-    #[deftly(has_memory_cost(indirect_size = "0"))] // estimate (it contains an Arc)
-    pub(crate) memquota: StreamAccount,
-}
-
 /// Data required for handling an incoming stream request.
 #[cfg(feature = "hs-service")]
 #[derive(educe::Educe)]
@@ -818,7 +778,7 @@ impl Reactor {
         #[cfg(feature = "conflux")]
         self.try_dequeue_ooo_msgs().await?;
 
-        let mut actions = select_biased! {
+        let mut events = select_biased! {
             res = self.command.next() => {
                 let cmd = unwrap_or_shutdown!(self, res, "command channel drop")?;
                 return ControlHandler::new(self).handle_cmd(cmd);
@@ -833,26 +793,26 @@ impl Reactor {
             // circuit is ready for sending.
             ret = self.control.next() => {
                 let msg = unwrap_or_shutdown!(self, ret, "control drop")?;
-                smallvec![CircuitAction::HandleControl(msg)]
+                smallvec![CircuitEvent::HandleControl(msg)]
             },
-            res = self.circuits.next_circ_action(&self.runtime).fuse() => res?,
+            res = self.circuits.next_circ_event(&self.runtime).fuse() => res?,
         };
 
-        // Put the actions into the order that we need to execute them in.
+        // Put the events into the order that we need to execute them in.
         //
-        // (Yes, this _does_ have to be a stable sort.  Not all actions may be freely re-ordered
+        // (Yes, this _does_ have to be a stable sort.  Not all events may be freely re-ordered
         // with respect to one another.)
-        actions.sort_by_key(|a| a.order_within_batch());
+        events.sort_by_key(|a| a.order_within_batch());
 
-        for action in actions {
-            let cmd = match action {
-                CircuitAction::RunCmd { leg, cmd } => Some(RunOnceCmd::Single(
+        for event in events {
+            let cmd = match event {
+                CircuitEvent::RunCmd { leg, cmd } => Some(RunOnceCmd::Single(
                     RunOnceCmdInner::from_circuit_cmd(leg, cmd),
                 )),
-                CircuitAction::HandleControl(ctrl) => ControlHandler::new(self)
+                CircuitEvent::HandleControl(ctrl) => ControlHandler::new(self)
                     .handle_msg(ctrl)?
                     .map(RunOnceCmd::Single),
-                CircuitAction::HandleCell { leg, cell } => {
+                CircuitEvent::HandleCell { leg, cell } => {
                     let circ = self
                         .circuits
                         .leg_mut(leg)
@@ -875,23 +835,23 @@ impl Reactor {
                         Some(cmd)
                     }
                 }
-                CircuitAction::RemoveLeg { leg, reason } => {
+                CircuitEvent::RemoveLeg { leg, reason } => {
                     Some(RunOnceCmdInner::RemoveLeg { leg, reason }.into())
                 }
-                CircuitAction::PaddingAction {
-                    leg,
-                    padding_action,
-                } => {
+                CircuitEvent::PaddingAction { leg, padding_event } => {
                     cfg_if! {
                         if #[cfg(feature = "circ-padding")] {
-                            Some(RunOnceCmdInner::PaddingAction { leg, padding_action }.into())
+                            Some(RunOnceCmdInner::PaddingAction { leg, padding_event }.into())
                         } else {
-                            // If padding isn't enabled, we never generate a padding action,
+                            // If padding isn't enabled, we never generate a padding event,
                             // so we can be sure this case will never be called.
-                            void::unreachable(padding_action.0);
+                            void::unreachable(padding_event.0);
                         }
                     }
                 }
+                CircuitEvent::ProtoViolation { err } => {
+                    return Err(err.into());
+                }
             };
 
             if let Some(cmd) = cmd {
@@ -1245,15 +1205,10 @@ impl Reactor {
                 self.ooo_msgs.push(entry);
             }
             #[cfg(feature = "circ-padding")]
-            RunOnceCmdInner::PaddingAction {
-                leg,
-                padding_action,
-            } => {
+            RunOnceCmdInner::PaddingAction { leg, padding_event } => {
                 // TODO: If we someday move back to having a per-circuit reactor,
-                // this action would logically belong there, not on the tunnel reactor.
-                self.circuits
-                    .run_padding_action(leg, padding_action)
-                    .await?;
+                // this event would logically belong there, not on the tunnel reactor.
+                self.circuits.run_padding_event(leg, padding_event).await?;
             }
         }
 
