@@ -1,22 +1,21 @@
 //! Module exposing types for representing circuits in the tunnel reactor.
 
-mod cell_sender;
 pub(crate) mod circhop;
 pub(super) mod create;
 pub(super) mod extender;
 
 use crate::channel::Channel;
-use crate::circuit::UniqId;
-use crate::circuit::celltypes::{ClientCircChanMsg, CreateResponse};
+use crate::circuit::cell_sender::CircuitCellSender;
+use crate::circuit::celltypes::CreateResponse;
 use crate::circuit::circhop::HopSettings;
+use crate::circuit::{CircuitRxReceiver, UniqId};
 use crate::client::circuit::handshake::{BoxedClientLayer, HandshakeRole};
 use crate::client::circuit::padding::{
     self, PaddingController, PaddingEventStream, QueuedCellPaddingInfo,
 };
-use crate::client::circuit::{CircuitRxReceiver, MutableState};
+use crate::client::circuit::{ClientCircChanMsg, MutableState};
 use crate::client::circuit::{TimeoutEstimator, path};
 use crate::client::reactor::MetaCellDisposition;
-use crate::client::reactor::circuit::cell_sender::CircuitCellSender;
 use crate::congestion::CongestionSignals;
 use crate::congestion::sendme;
 use crate::crypto::binding::CircuitBinding;
@@ -29,11 +28,11 @@ use crate::crypto::handshake::ntor::{NtorClient, NtorPublicKey};
 use crate::crypto::handshake::ntor_v3::{NtorV3Client, NtorV3PublicKey};
 use crate::crypto::handshake::{ClientHandshake, KeyGenerator};
 use crate::memquota::{CircuitAccount, SpecificAccount as _, StreamAccount};
-use crate::stream::StreamMpscReceiver;
 use crate::stream::cmdcheck::{AnyCmdChecker, StreamStatus};
 use crate::stream::flow_ctrl::state::StreamRateLimit;
 use crate::stream::flow_ctrl::xon_xoff::reader::DrainRateRequest;
 use crate::stream::queue::{StreamQueueSender, stream_queue};
+use crate::stream::{StreamMpscReceiver, msg_streamid};
 use crate::streammap;
 use crate::tunnel::TunnelScopedCircId;
 use crate::util::err::ReactorError;
@@ -56,9 +55,8 @@ use tor_memquota::mq_queue::{ChannelSpec as _, MpscSpec};
 use futures::SinkExt as _;
 use oneshot_fused_workaround as oneshot;
 use postage::watch;
-use safelog::sensitive as sv;
 use tor_rtcompat::{DynTimeProvider, SleepProvider as _};
-use tracing::{debug, trace, warn};
+use tracing::{debug, instrument, trace, warn};
 
 use super::{
     CellHandlers, CircuitHandshake, CloseStreamBehavior, ReactorResultChannel, SendRelayCell,
@@ -77,6 +75,7 @@ use extender::HandshakeAuxDataHandler;
 
 #[cfg(feature = "hs-service")]
 use {
+    crate::circuit::CircSyncView,
     crate::client::stream::{InboundDataCmdChecker, IncomingStreamRequest},
     tor_cell::relaycell::msg::Begin,
 };
@@ -87,18 +86,10 @@ use {
     crate::tunnel::TunnelId,
 };
 
-pub(super) use circhop::{CircHop, CircHopList};
+#[cfg(not(feature = "flowctl-cc"))]
+use crate::stream::STREAM_READER_BUFFER;
 
-/// Initial value for outbound flow-control window on streams.
-pub(crate) const SEND_WINDOW_INIT: u16 = 500;
-/// Initial value for inbound flow-control window on streams.
-pub(crate) const RECV_WINDOW_INIT: u16 = 500;
-/// Size of the buffer used between the reactor and a `StreamReader`.
-///
-/// FIXME(eta): We pick 2× the receive window, which is very conservative (we arguably shouldn't
-///             get sent more than the receive window anyway!). We might do due to things that
-///             don't count towards the window though.
-pub(crate) const STREAM_READER_BUFFER: usize = (2 * RECV_WINDOW_INIT) as usize;
+pub(super) use circhop::{CircHop, CircHopList};
 
 /// A circuit "leg" from a tunnel.
 ///
@@ -314,7 +305,7 @@ impl Circuit {
         }
 
         let cell = SendRelayCell {
-            hop,
+            hop: Some(hop),
             early: false,
             cell,
         };
@@ -417,6 +408,7 @@ impl Circuit {
     /// NOTE: the reactor should not call this function directly, only via
     /// [`ConfluxSet::send_relay_cell_on_leg`](super::conflux::ConfluxSet::send_relay_cell_on_leg),
     /// which will reroute the message, if necessary to the primary leg.
+    #[instrument(level = "trace", skip_all)]
     pub(super) async fn send_relay_cell(&mut self, msg: SendRelayCell) -> Result<()> {
         self.send_relay_cell_inner(msg, None).await
     }
@@ -426,6 +418,7 @@ impl Circuit {
     ///
     /// If `padding_info` is None, `msg` must be non-padding: we report it as such to the
     /// padding controller.
+    #[instrument(level = "trace", skip_all)]
     async fn send_relay_cell_inner(
         &mut self,
         msg: SendRelayCell,
@@ -450,6 +443,7 @@ impl Circuit {
         let runtime = self.runtime.clone();
         let c_t_w = sendme::cmd_counts_towards_windows(msg.cmd());
         let stream_id = msg.stream_id();
+        let hop = hop.expect("missing hop in client SendRelayCell?!");
         let circhop = self.hops.get_mut(hop).ok_or(Error::NoSuchHop)?;
 
         // We might be out of capacity entirely; see if we are about to hit a limit.
@@ -474,7 +468,7 @@ impl Circuit {
         //            the whole circuit (e.g. by returning an error).
         let (msg, tag) = Self::encode_relay_cell(
             &mut self.crypto_out,
-            circhop.relay_format(),
+            circhop.relay_cell_format(),
             hop,
             early,
             msg,
@@ -482,7 +476,7 @@ impl Circuit {
         // The cell counted for congestion control, inform our algorithm of such and pass down the
         // tag for authenticated SENDMEs.
         if c_t_w {
-            circhop.ccontrol_mut().note_data_sent(&runtime, &tag)?;
+            circhop.ccontrol().note_data_sent(&runtime, &tag)?;
         }
 
         // Remember that we've enqueued this cell.
@@ -589,7 +583,7 @@ impl Circuit {
         let send_circ_sendme = if c_t_w {
             self.hop_mut(hopnum)
                 .ok_or_else(|| Error::CircProto("Sendme from nonexistent hop".into()))?
-                .ccontrol_mut()
+                .ccontrol()
                 .note_data_received()?
         } else {
             false
@@ -605,7 +599,7 @@ impl Circuit {
             let sendme = Sendme::from(tag);
             let cell = AnyRelayMsgOuter::new(None, sendme.into());
             circ_cmds.push(CircuitCmd::Send(SendRelayCell {
-                hop: hopnum,
+                hop: Some(hopnum),
                 early: false,
                 cell,
             }));
@@ -618,7 +612,7 @@ impl Circuit {
                         hopnum
                     ))
                 })?
-                .ccontrol_mut()
+                .ccontrol()
                 .note_sendme_sent()?;
         }
 
@@ -753,7 +747,7 @@ impl Circuit {
         if let Some(cell) = hop.maybe_send_xoff(streamid)? {
             let cell = AnyRelayMsgOuter::new(Some(streamid), cell.into());
             let cell = SendRelayCell {
-                hop: hopnum,
+                hop: Some(hopnum),
                 early: false,
                 cell,
             };
@@ -898,7 +892,7 @@ impl Circuit {
             //
             // This means it's very important not to call this function while any of the hop's
             // stream map mutex is held.
-            let view = ClientCircSyncView::new(&self.hops);
+            let view = CircSyncView::new_client(ClientCircSyncView::new(&self.hops));
 
             match handler.filter.as_mut().disposition(&ctx, &view)? {
                 Accept => {}
@@ -906,7 +900,7 @@ impl Circuit {
                 RejectRequest(end) => {
                     let end_msg = AnyRelayMsgOuter::new(Some(stream_id), end.into());
                     let cell = SendRelayCell {
-                        hop: hop_num,
+                        hop: Some(hop_num),
                         early: false,
                         cell: end_msg,
                     };
@@ -918,7 +912,7 @@ impl Circuit {
         // TODO: Sadly, we need to look up `&mut hop` yet again,
         // since we needed to pass `&self.hops` by reference to our filter above. :(
         let hop = self.hops.get_mut(hop_num).ok_or(Error::CircuitClosed)?;
-        let relay_cell_format = hop.relay_format();
+        let relay_cell_format = hop.relay_cell_format();
 
         let memquota = StreamAccount::new(&self.memquota)?;
 
@@ -955,7 +949,7 @@ impl Circuit {
         let outcome = Pin::new(&mut handler.incoming_sender).try_send(StreamReqInfo {
             req,
             stream_id,
-            hop: (leg, hop_num).into(),
+            hop: Some((leg, hop_num).into()),
             msg_tx,
             receiver,
             rate_limit_stream: rate_limit_rx,
@@ -977,7 +971,7 @@ impl Circuit {
                 );
 
                 let cell = SendRelayCell {
-                    hop: hop_num,
+                    hop: Some(hop_num),
                     early: false,
                     cell: end_msg,
                 };
@@ -1362,6 +1356,7 @@ impl Circuit {
     }
 
     /// Handle a RELAY_SENDME cell on this circuit with stream ID 0.
+    #[instrument(level = "trace", skip_all)]
     pub(super) fn handle_sendme(
         &mut self,
         hopnum: HopNum,
@@ -1382,7 +1377,7 @@ impl Circuit {
                 // but we don't support those any longer.
                  Error::CircProto("missing tag on circuit sendme".into()))?;
         // Update the CC object that we received a SENDME along with possible congestion signals.
-        hop.ccontrol_mut()
+        hop.ccontrol()
             .note_sendme_received(&runtime, tag, signals)?;
         Ok(None)
     }
@@ -1401,6 +1396,7 @@ impl Circuit {
     /// check whether the channel is ready to receive messages (`self.channel.poll_ready`), and
     /// ideally use this to implement backpressure (such that you do not read from other sources
     /// that would send here while you know you're unable to forward the messages on).
+    #[instrument(level = "trace", skip_all)]
     async fn send_msg(
         &mut self,
         msg: AnyChanMsg,
@@ -1460,6 +1456,7 @@ impl Circuit {
     }
 
     /// Close the specified stream
+    #[instrument(level = "trace", skip_all)]
     pub(super) async fn close_stream(
         &mut self,
         hop_num: HopNum,
@@ -1689,7 +1686,7 @@ impl Circuit {
     ) -> Result<()> {
         use tor_cell::relaycell::msg::Drop as DropMsg;
         let msg = SendRelayCell {
-            hop: target_hop,
+            hop: Some(target_hop),
             // TODO circpad: we will probably want padding machines that can send EARLY cells.
             early: false,
             cell: AnyRelayMsgOuter::new(None, DropMsg::default().into()),
@@ -1731,23 +1728,6 @@ enum CircPaddingDisposition {
     TreatQueuedCellAsPadding,
 }
 
-/// Return the stream ID of `msg`, if it has one.
-///
-/// Returns `Ok(None)` if `msg` is a meta cell.
-fn msg_streamid(msg: &UnparsedRelayMsg) -> Result<Option<StreamId>> {
-    let cmd = msg.cmd();
-    let streamid = msg.stream_id();
-    if !cmd.accepts_streamid_val(streamid) {
-        return Err(Error::CircProto(format!(
-            "Invalid stream ID {} for relay command {}",
-            sv(StreamId::get_or_zero(streamid)),
-            msg.cmd()
-        )));
-    }
-
-    Ok(streamid)
-}
-
 impl Drop for Circuit {
     fn drop(&mut self) {
         let _ = self.channel.close_circuit(self.channel_id);
