@@ -1,5 +1,10 @@
 //! Conflux-related functionality
 
+// TODO: replace Itertools::exactly_one() with a stdlib equivalent when there is one.
+//
+// See issue #48919 <https://github.com/rust-lang/rust/issues/48919>
+#![allow(unstable_name_collisions)]
+
 #[cfg(feature = "conflux")]
 pub(crate) mod msghandler;
 
@@ -12,28 +17,30 @@ use itertools::Itertools;
 use itertools::structs::ExactlyOneError;
 use smallvec::{SmallVec, smallvec};
 use tor_rtcompat::{SleepProvider as _, SleepProviderExt as _};
-use tracing::{info, trace, warn};
+use tracing::{info, instrument, trace, warn};
 
 use tor_cell::relaycell::AnyRelayMsgOuter;
 use tor_error::{Bug, bad_api_usage, internal};
 use tor_linkspec::HasRelayIds as _;
 
 use crate::circuit::UniqId;
+use crate::circuit::circhop::SendRelayCell;
 use crate::client::circuit::TunnelMutableState;
 #[cfg(feature = "circ-padding")]
 use crate::client::circuit::padding::PaddingEvent;
 use crate::client::circuit::path::HopDetail;
-use crate::client::streammap;
 use crate::conflux::cmd_counts_towards_seqno;
 use crate::conflux::msghandler::{ConfluxStatus, RemoveLegReason};
 use crate::congestion::params::CongestionWindowParams;
 use crate::crypto::cell::HopNum;
+use crate::streammap;
 use crate::tunnel::TunnelId;
 use crate::util::err::ReactorError;
 use crate::util::poll_all::PollAll;
+use crate::util::tunnel_activity::TunnelActivity;
 
 use super::circuit::CircHop;
-use super::{Circuit, CircuitAction, SendRelayCell};
+use super::{Circuit, CircuitEvent};
 
 #[cfg(feature = "conflux")]
 use {
@@ -51,14 +58,14 @@ use {
 const MAX_CONFLUX_LEGS: usize = 16;
 
 /// The number of futures we add to the per-circuit [`PollAll`] future in
-/// [`ConfluxSet::next_circ_action`].
+/// [`ConfluxSet::next_circ_event`].
 ///
 /// Used for the SmallVec size estimate;
 const NUM_CIRC_FUTURES: usize = 2;
 
-/// The expected number of circuit actions to be returned from
-/// [`ConfluxSet::next_circ_action`]
-const CIRC_ACTION_COUNT: usize = MAX_CONFLUX_LEGS * NUM_CIRC_FUTURES;
+/// The expected number of circuit events to be returned from
+/// [`ConfluxSet::next_circ_event`]
+const CIRC_EVENT_COUNT: usize = MAX_CONFLUX_LEGS * NUM_CIRC_FUTURES;
 
 /// A set with one or more circuits.
 ///
@@ -284,6 +291,7 @@ impl ConfluxSet {
     /// We do not yet support resumption. See [2.4.3. Closing circuits] in prop329.
     ///
     /// [2.4.3. Closing circuits]: https://spec.torproject.org/proposals/329-traffic-splitting.html#243-closing-circuits
+    #[instrument(level = "trace", skip_all)]
     pub(super) fn remove(&mut self, leg: UniqId) -> Result<Circuit, ReactorError> {
         let circ = self.remove_unchecked(leg)?;
 
@@ -420,6 +428,14 @@ impl ConfluxSet {
         self.legs.iter()
     }
 
+    /// Return the most active [`TunnelActivity`] for any leg of this `ConfluxSet`.
+    pub(super) fn tunnel_activity(&self) -> TunnelActivity {
+        self.circuits()
+            .map(|c| c.hops.tunnel_activity())
+            .max()
+            .unwrap_or_else(TunnelActivity::never_used)
+    }
+
     /// Add legs to the this conflux set.
     ///
     /// Returns an error if any of the legs is invalid.
@@ -672,7 +688,7 @@ impl ConfluxSet {
         let switch = ConfluxSwitch::new(seqno_delta);
         let cell = AnyRelayMsgOuter::new(None, switch.into());
         Ok(Some(SendRelayCell {
-            hop: join_point,
+            hop: Some(join_point),
             early: false,
             cell,
         }))
@@ -810,7 +826,7 @@ impl ConfluxSet {
         Ok(!join_circhop.ccontrol().can_send())
     }
 
-    /// Returns whether [`next_circ_action`](Self::next_circ_action)
+    /// Returns whether [`next_circ_event`](Self::next_circ_event)
     /// should avoid polling the join point streams entirely.
     #[cfg(feature = "conflux")]
     fn should_skip_join_point(&self) -> Result<bool, Bug> {
@@ -886,7 +902,7 @@ impl ConfluxSet {
         Ok(should_skip)
     }
 
-    /// Returns the next ready [`CircuitAction`],
+    /// Returns the next ready [`CircuitEvent`],
     /// obtained from processing the incoming/outgoing messages on all the circuits in this set.
     ///
     /// Will return an error if there are no circuits in this set,
@@ -894,10 +910,11 @@ impl ConfluxSet {
     ///
     /// This is cancellation-safe.
     #[allow(clippy::unnecessary_wraps)] // Can return Err if conflux is enabled
-    pub(super) async fn next_circ_action(
+    #[instrument(level = "trace", skip_all)]
+    pub(super) async fn next_circ_event(
         &mut self,
         runtime: &tor_rtcompat::DynTimeProvider,
-    ) -> Result<SmallVec<[CircuitAction; CIRC_ACTION_COUNT]>, crate::Error> {
+    ) -> Result<SmallVec<[CircuitEvent; CIRC_EVENT_COUNT]>, crate::Error> {
         // Avoid polling the streams on the join point if our primary
         // leg is blocked on cc
         cfg_if::cfg_if! {
@@ -924,12 +941,12 @@ impl ConfluxSet {
         // into `PollAll` before future B, the result of future A will come
         // before future B's result in the result list returned by poll_all.await.
         //
-        // This means that the actions corresponding to the first circuit in the tunnel
-        // will be executed first, followed by the actions issued by the next circuit,
-        // and s on.
+        // This means that the events corresponding to the first circuit in the tunnel
+        // will be executed first, followed by the events issued by the next circuit,
+        // and so on.
         //
         let mut poll_all =
-            PollAll::<MAX_CONFLUX_LEGS, SmallVec<[CircuitAction; NUM_CIRC_FUTURES]>>::new();
+            PollAll::<MAX_CONFLUX_LEGS, SmallVec<[CircuitEvent; NUM_CIRC_FUTURES]>>::new();
 
         for leg in &mut self.legs {
             let unique_id = leg.unique_id();
@@ -951,14 +968,25 @@ impl ConfluxSet {
             // implementation currently uses Circuit Build Timeout).
             let conflux_hs_timeout = leg.conflux_hs_timeout();
 
-            let mut poll_all_circ = PollAll::<NUM_CIRC_FUTURES, CircuitAction>::new();
+            let mut poll_all_circ = PollAll::<NUM_CIRC_FUTURES, CircuitEvent>::new();
 
             let input = leg.input.next().map(move |res| match res {
-                Some(cell) => CircuitAction::HandleCell {
-                    leg: unique_id,
-                    cell,
+                Some(msg) => match msg.try_into() {
+                    Ok(cell) => CircuitEvent::HandleCell {
+                        leg: unique_id,
+                        cell,
+                    },
+                    // A message outside our restricted set is either a fatal internal error or
+                    // a protocol violation somehow so shutdown.
+                    //
+                    // TODO(relay): We have this spec ticket open about this behavior:
+                    // https://gitlab.torproject.org/tpo/core/torspec/-/issues/385. It is plausible
+                    // that we decide to either keep this circuit close behavior or close the
+                    // entire channel in this case. Resolution of the above ticket needs to fix
+                    // this part.
+                    Err(e) => CircuitEvent::ProtoViolation { err: e },
                 },
-                None => CircuitAction::RemoveLeg {
+                None => CircuitEvent::RemoveLeg {
                     leg: unique_id,
                     reason: RemoveLegReason::ChannelClosed,
                 },
@@ -998,21 +1026,21 @@ impl ConfluxSet {
                         // We await an infinitely pending future so that we don't
                         // immediately return a `None` in the `select_biased!` below.
                         // We'd rather wait on `input.next()` than immediately return with
-                        // no `CircuitAction`, which could put the reactor into a spin loop.
+                        // no `CircuitEvent`, which could put the reactor into a spin loop.
                         let () = std::future::pending().await;
                         unreachable!();
                     }
                 }
             };
 
-            poll_all_circ.push(next_ready_stream.map(move |cmd| CircuitAction::RunCmd {
+            poll_all_circ.push(next_ready_stream.map(move |cmd| CircuitEvent::RunCmd {
                 leg: unique_id,
                 cmd,
             }));
 
             let mut next_padding_event_fut = leg.padding_event_stream.next();
 
-            // This selects between 3 actions that cannot be handled concurrently.
+            // This selects between 3 events that cannot be handled concurrently.
             //
             // If the conflux handshake times out, we need to remove the circuit leg
             // (any pending padding events or application stream data should be discarded;
@@ -1041,16 +1069,16 @@ impl ConfluxSet {
 
                             // Conflux handshake has timed out, time to remove this circuit leg,
                             // and notify the handshake initiator.
-                            smallvec![CircuitAction::RemoveLeg {
+                            smallvec![CircuitEvent::RemoveLeg {
                                 leg: unique_id,
                                 reason: RemoveLegReason::ConfluxHandshakeTimeout,
                             }]
                         }
-                        padding_action = next_padding_event_fut => {
-                            smallvec![CircuitAction::PaddingAction {
+                        padding_event = next_padding_event_fut => {
+                            smallvec![CircuitEvent::PaddingAction {
                                 leg: unique_id,
-                                padding_action:
-                                    padding_action.expect("PaddingEventStream, surprisingly, was terminated!"),
+                                padding_event:
+                                    padding_event.expect("PaddingEventStream, surprisingly, was terminated!"),
                             }]
                         }
                         ret = poll_all_circ.fuse() => ret,
@@ -1060,7 +1088,7 @@ impl ConfluxSet {
         }
 
         // Flatten the nested SmallVecs to simplify the calling code
-        // (which will handle all the returned actions sequentially).
+        // (which will handle all the returned events sequentially).
         Ok(poll_all.await.into_iter().flatten().collect())
     }
 
@@ -1081,6 +1109,7 @@ impl ConfluxSet {
     /// Encode `msg`, encrypt it, and send it to the 'hop'th hop.
     ///
     /// See [`Circuit::send_relay_cell`].
+    #[instrument(level = "trace", skip_all)]
     pub(super) async fn send_relay_cell_on_leg(
         &mut self,
         msg: SendRelayCell,
@@ -1088,10 +1117,11 @@ impl ConfluxSet {
     ) -> crate::Result<()> {
         let conflux_join_point = self.join_point.as_ref().map(|join_point| join_point.hop);
         let leg = if let Some(join_point) = conflux_join_point {
+            let hop = msg.hop.expect("missing hop in client SendRelayCell?!");
             // Conflux circuits always send multiplexed relay commands to
             // to the last hop (the join point).
             if cmd_counts_towards_seqno(msg.cell.cmd()) {
-                if msg.hop != join_point {
+                if hop != join_point {
                     // For leaky pipe, we must continue using the original leg
                     leg
                 } else {
@@ -1201,12 +1231,12 @@ impl ConfluxSet {
         Ok(self.legs.remove(idx))
     }
 
-    /// Perform some circuit-padding-based action on the specified circuit.
+    /// Perform some circuit-padding-based event on the specified circuit.
     #[cfg(feature = "circ-padding")]
-    pub(super) async fn run_padding_action(
+    pub(super) async fn run_padding_event(
         &mut self,
         circ_id: UniqId,
-        padding_action: PaddingEvent,
+        padding_event: PaddingEvent,
     ) -> crate::Result<()> {
         use PaddingEvent as E;
         let Some(circ) = self.leg_mut(circ_id) else {
@@ -1215,7 +1245,7 @@ impl ConfluxSet {
             return Ok(());
         };
 
-        match padding_action {
+        match padding_event {
             E::SendPadding(send_padding) => {
                 circ.send_padding(send_padding).await?;
             }
