@@ -3,10 +3,10 @@
 use serde::Deserialize;
 use std::{fmt::Debug, path::PathBuf, str::FromStr};
 use tor_config_path::{
-    addr::{CfgAddr, CfgAddrError},
     CfgPath, CfgPathError, CfgPathResolver,
+    addr::{CfgAddr, CfgAddrError},
 };
-use tor_general_addr::general;
+use tor_general_addr::general::{self, AddrParseError};
 
 use crate::HasClientErrorAction;
 
@@ -90,6 +90,9 @@ pub enum ResolveError {
     /// There was a path in the connect point that we couldn't resolve.
     #[error("Unable to resolve variables in path")]
     InvalidPath(#[from] CfgPathError),
+    ///  There was an address in the connect point that we couldn't parse.
+    #[error("Unable to parse address")]
+    UnparseableAddr(#[from] AddrParseError),
     /// There was an address in the connect point that we couldn't resolve.
     #[error("Unable to resolve variables in address")]
     InvalidAddr(#[from] CfgAddrError),
@@ -110,7 +113,7 @@ pub enum ResolveError {
     /// (This can only happen if somebody adds new variants to `general::SocketAddr`.)
     #[error("Address type not recognized")]
     AddressTypeNotRecognized,
-    /// The name of a file or unix socket address was a relative path.
+    /// The name of a file or AF_UNIX socket address was a relative path.
     #[error("Path was not absolute")]
     PathNotAbsolute,
 }
@@ -119,6 +122,7 @@ impl HasClientErrorAction for ResolveError {
         use crate::ClientErrorAction as A;
         match self {
             ResolveError::InvalidPath(e) => e.client_action(),
+            ResolveError::UnparseableAddr(e) => e.client_action(),
             ResolveError::InvalidAddr(e) => e.client_action(),
             ResolveError::PathNotString => A::Decline,
             ResolveError::AddressNotLoopback => A::Decline,
@@ -218,6 +222,7 @@ pub(crate) enum BuiltinVariant {
 
 #[derive(Deserialize, Clone, Debug)]
 #[serde(bound = "R::Path : Deserialize<'de>, AddrWithStr<R::SocketAddr> : Deserialize<'de>")]
+#[allow(clippy::missing_docs_in_private_items)]
 pub(crate) struct Connect<R: Addresses> {
     /// The address of the socket at which the client should try to reach the RPC server,
     /// and which the RPC server should bind.
@@ -260,7 +265,7 @@ impl Connect<Resolved> {
         use general::SocketAddr::{Inet, Unix};
         match (self.socket.as_ref(), &self.auth) {
             (Inet(addr), _) if !addr.ip().is_loopback() => {
-                return Err(ResolveError::AddressNotLoopback)
+                return Err(ResolveError::AddressNotLoopback);
             }
             (Inet(_), Auth::None) => return Err(ResolveError::AuthNotCompatible),
             (_, Auth::Unrecognized(_)) => return Err(ResolveError::AuthNotRecognized),
@@ -339,7 +344,7 @@ impl Auth<Resolved> {
 #[derive(Clone, Debug)]
 struct Unresolved;
 impl Addresses for Unresolved {
-    type SocketAddr = CfgAddr;
+    type SocketAddr = String;
     type Path = CfgPath;
 }
 
@@ -365,7 +370,7 @@ where
     /// The string representation of the address.
     ///
     /// For inet addresses, this is the value that appeared in the configuration.
-    /// For unix addresses, this is the value that appeared in the configuration,
+    /// For unix domain sockets, this is the value that appeared in the configuration,
     /// after shell expansion.
     string: String,
     /// The address itself.
@@ -382,13 +387,14 @@ where
         self.string.as_str()
     }
 }
-impl AddrWithStr<CfgAddr> {
-    /// Convert an `AddrWithStr<CfgAddr>` into its substituted form.
+impl AddrWithStr<String> {
+    /// Convert an `AddrWithStr<String>` into its substituted form.
     pub(crate) fn resolve(
         &self,
         resolver: &CfgPathResolver,
     ) -> Result<AddrWithStr<general::SocketAddr>, ResolveError> {
         let AddrWithStr { string, addr } = self;
+        let addr: CfgAddr = addr.parse()?;
         let substituted = addr.substitutions_will_apply();
         let addr = addr.address(resolver)?;
         let string = if substituted {
@@ -437,7 +443,7 @@ mod test {
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
     #![allow(clippy::unwrap_used)]
-    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::unchecked_time_subtraction)]
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
